from multiprocessing import Pool
import argparse
import os
import sys
import uuid
from glob import glob
from errno import ENOENT as FILENOTFOUNDCODE

import numpy as np
import pysam
import pyBigWig
import SharedArray as sha


def accept_as_input(bamfp):
    """Test proposed input bam file paths for some things."""
    pathto, name = os.path.split(bamfp)
    if pathto == "": pathto = "."
    assert os.path.exists(pathto), "This path [{}] doesn't exist".format(pathto)

    bamfp = os.path.join(pathto, name)
    if not os.path.isfile(bamfp):
        return False, "Designated input file {} doesn't seem to exist".format(bamfp)

    if not os.path.exists(bamfp + ".bai"):
        return False, "Input file {} doesn't seem to be indexed (I can't find the relevant .bai file), please index it with 'samtools index'".format(
            bamfp)

    return True, None


class FlagFilter():
    """This class instantiates a filter based on the args.
    The idea is to only ever expose one filtering fuction for the reads and
    not to have unnecessary elifs and clutter. It is a bit hacky but fun.
    """

    def __init__(self, args):
        """Set self.filter_function based on the arguments."""
        if args.filter_lc and args.filter_uc:
            self.filter_func = self.both(args)
        elif args.filter_lc:
            self.filter_func = self.lc(args)
        elif args.filter_uc:
            self.filter_func = self.uc(args)
        else:
            self.filter_func = lambda x: True

    def both(self, args):
        """If both filters exists."""
        flc = args.filter_lc
        fuc = args.filter_uc
        return lambda x: ((x & flc) == flc) and ((x & fuc) == 0)

    def lc(self, args):
        """Only lowercase filter is set."""
        flc = args.filter_lc
        return lambda x: (x & flc) == flc

    def uc(self, args):
        """Only uppercase filter is set."""
        fuc = args.filter_uc
        return lambda x: ((x & fuc) == 0)


class FragFilter():
    """This class instantiates a filter based on the args.
    The idea is to only ever expose one filtering fuction for the reads and
    not to have unnecessary elifs and clutter.
    """

    def __init__(self, args):
        """Set self.filter_function based on the arguments."""
        if args.t_len_lower and args.t_len_upper:
            self.filter_func = self.both(args)
        elif args.t_len_lower:
            self.filter_func = self.lc(args)
        elif args.t_len_upper:
            self.filter_func = self.uc(args)
        else:
            self.filter_func = lambda x: True

    def both(self, args):
        """If both filters exists."""
        tll = args.t_len_lower
        tlu = args.t_len_upper
        return lambda x: (x >= tll) and (x <= tlu)

    def lc(self, args):
        """Only lower filter is set."""
        tll = args.t_len_lower
        return lambda x: (x >= tll)

    def uc(self, args):
        """Only upper filter is set."""
        tlu = args.t_len_upper
        return lambda x: (x <= tlu)


def get_extension_sites(args, chrom):
    """Get reads for one chromosome into dp and dn.
    Returns dp for reads in the + strand and dn for reads in the - strand
    If extsize is a digit, dn reads are shifted 5-->3 by extsize.
    That way we can just extend every read in one direction in the next steps.
    """
    chromsize = args.chrom_size.get(chrom)
    _extend = args.extsize

    bams = [pysam.AlignmentFile(bamfp) for bamfp in args.bam]
    # ff is a function that gets instantiated based on
    # the arguments. It returns True for acceptable reads
    f = FlagFilter(args)
    # the flag filter
    ff = f.filter_func

    fr = FragFilter(args)
    # the fragment length filter
    ffr = fr.filter_func

    # the quality filter
    if args.q:
        fq = lambda x: x >= args.q
    else:
        fq = lambda x: True

    # p strand constant
    ps_const = args.shift + args.shift_p
    # n strand constant
    if 'fragment' == _extend:
        ns_const = -args.shift - args.shift_n
    else:
        ns_const = -args.shift - args.shift_n - _extend

        # This will go through all the reads,
    # and with the reads that pass the various filters
    # it will create a dictionary
    # where d = { position: NumberofExtentionSites }
    dp = {}
    dn = {}
    for enum, bam in enumerate(bams):
        try:
            for read in bam.fetch(chrom, 0, chromsize, multiple_iterators=True):
                if ff(read.flag) and ffr(abs(read.template_length)) and fq(read.mapping_quality):
                    if read.is_reverse:
                        # http://pysam.readthedocs.io/en/latest/api.html
                        # reference_end points to one past the last aligned residue.
                        # Returns None if not available (read is unmapped or no cigar alignment present).
                        dpos = read.reference_end + ns_const
                        dn[dpos] = dn.get(dpos, 0) + 1
                    else:
                        dpos = read.reference_start + ps_const
                        dp[dpos] = dp.get(dpos, 0) + 1
        except ValueError:
            args.vprint(
                "{} (found in the chrom-info file) doesn't exist in the bam file {} ".format(chrom, args.bam[enum]),
                file=sys.stderr)
        bam.close()
    args.vprint(
        "Finished reading from {}. Found {} reads in the + strand and {} in the -".format(chrom, sum(dp.values()),
                                                                                          sum(dn.values())),
        file=sys.stdout)
    return dp, dn


def cuts_d_extend(d, extend):
    """Convert d into dee."""
    ed = {k + extend + 1: v * -1 for k, v in d.items()}
    dee = {k: d.get(k, 0) + ed.get(k, 0) for k in set(d.keys()).union(set(ed.keys()))}
    return dee


def yield_bedgraph_fields(dee):
    """From dictionary of position:signalchange to bedgraph fields.
    Takes dee which stores positions and increase/decrease in signal
    and outputs (start, end, value) of signal. The core of the "algorithm"
    """
    ipoints = iter(sorted(dee.keys()))
    start = next(ipoints)
    val = dee.get(start, 0)

    for nextpoint in ipoints:
        valchange = dee.get(nextpoint, 0)
        if valchange == 0:
            continue
        else:
            if val > 0 and nextpoint > 0:
                yield (max([0, start]), nextpoint, val)
            elif val < 0:
                vprint("value under zeros shouldnt happen", file=sys.stderr)
            start = nextpoint
            val += valchange


def poolinit(arguments):
    """Initialize the pool and make globals among all the processes.
    http://stackoverflow.com/questions/25557686/python-sharing-a-lock-between-processes
    """
    global gargs
    gargs = arguments


def to_shm(dee, schmeckle):
    """Convert dee to bedgraph fields and write to shared memory."""
    try:
        starts, ends, values = zip(*yield_bedgraph_fields(dee))
        size = len(starts)

        sha_starts = sha.create(schmeckle.format("starts"), size, dtype=np.int64)
        sha_ends = sha.create(schmeckle.format("ends"), size, dtype=np.int64)
        sha_values = sha.create(schmeckle.format("values"), size, dtype=np.float64)

        sha_starts[:] = starts
        sha_ends[:] = ends
        sha_values[:] = values

        del starts, ends, values
    except:
        return 1
    return 0


def worker(chrom):
    """A pool worker that handles one chromosome.
    Gets the reads with get_extension_sites
    makes a unique schmeckle string in /dev/shm with the SharedArray library
    Then gets bedgraph fields and stores them in the shared memory,
    in the unique schmeckle filename.
    Returns the chrom and schmeckle so that the parent process
    knows which data to retrieve for this chrom.
    Precisely returns [chrom, smek1, smek2], smek1 and 2 cant be None
    """
    dp, dn = get_extension_sites(gargs, chrom)
    myuuid = uuid.uuid1()
    _extend = gargs.extsize

    schmeckle = "{}bam2bw_{}_{}_{}_{}".format(gargs.shm_backend, myuuid, "{}", chrom, "{}")
    ret = [chrom, None, None]
    if "fragment" == _extend:
        if not {} == dp:
            schmeckle1 = schmeckle.format("c", "{}")
            assert sum(dp.values()), sum(dn.values())
            ed = {k: v * -1 for k, v in dn.items()}
            dee = {k: dp.get(k, 0) + ed.get(k, 0) for k in set(dp.keys()).union(set(ed.keys()))}

            err = to_shm(dee, schmeckle1)
            if not err:
                ret[1] = schmeckle1
    else:
        if gargs.split_strands:
            schmeckle1 = schmeckle.format("p", "{}")
            schmeckle2 = schmeckle.format("n", "{}")
            if not {} == dp:
                err = to_shm(cuts_d_extend(dp, _extend), schmeckle1)
                if not err:
                    ret[1] = schmeckle1

            if not {} == dn:
                err = to_shm(cuts_d_extend(dn, _extend), schmeckle2)
                if not err:
                    ret[2] = schmeckle2
        else:
            dc = {k: dp.get(k, 0) + dn.get(k, 0) for k in set(dp.keys()).union(dn.keys())}
            schmeckle1 = schmeckle.format("c", "{}")
            # del dp, dn

            if not {} == dc:
                err = to_shm(cuts_d_extend(dc, _extend), schmeckle1)
                if not err:
                    ret[1] = schmeckle1

    gargs.vprint("Wrote {} to shared mem".format(chrom), file=sys.stdout)
    return ret


def shm_to_pybw_manager(chrom, schmeckle, bwl, args):
    """Organize the sharedMemory to bigwig.

    This is called by the 'manager' process, the one that makes the pool
    and handles the results.
    """
    smek1, smek2 = schmeckle
    if args.split_strands and len(bwl) == 2:
        if smek1:
            shm_to_pybw(chrom, smek1, bwl[0], args)
        if smek2:
            shm_to_pybw(chrom, smek2, bwl[1], args)

    elif (not args.split_strands) and len(bwl) == 1:
        if smek1:
            shm_to_pybw(chrom, smek1, bwl[0], args)
    else:
        sys.exit("CATASTROPHIC FAILURE.")

    return 0


def shm_to_pybw(chrom, schmeckle, bw, largs):
    """Read data from the shared memorry and put them in the pybw object."""
    try:
        sha_starts = sha.attach(schmeckle.format("starts"))
        sha_ends = sha.attach(schmeckle.format("ends"))
        sha_values = sha.attach(schmeckle.format("values"))
        mysize = len(sha_starts)
        bw.addEntries([chrom] * mysize, sha_starts, ends=sha_ends, values=sha_values)
        largs.vprint("Wrote {} entries in chromosome {}".format(mysize, chrom), file=sys.stdout)

        sha.delete(schmeckle.format("starts"))
        sha.delete(schmeckle.format("ends"))
        sha.delete(schmeckle.format("values"))
    except OSError as e:
        if e.errno == FILENOTFOUNDCODE:
            # do your FileNotFoundError code here
            largs.vprint("I expected to find this but couldn't: {}".format(schmeckle.format("{}")), file=sys.stderr)
            pass
        else:
            raise


def bamtobw(args):
    """The master process.
    Handles the pybiwwig object, makes the pool,
    reads the results from the shared memory, puts them in the pybw object,
    then closes it.
    """
    if args.split_strands:
        bw_p = pyBigWig.open(args.bw_p_path, "w")
        bw_p.addHeader(sorted(list(args.chrom_size.items())), maxZooms=args.maxzooms)
        bw_n = pyBigWig.open(args.bw_n_path, "w")
        bw_n.addHeader(sorted(list(args.chrom_size.items())), maxZooms=args.maxzooms)
        bwl = [bw_p, bw_n]

    else:
        bw = pyBigWig.open(args.bw_path, "w")
        bw.addHeader(sorted(list(args.chrom_size.items())), maxZooms=args.maxzooms)
        bwl = [bw]

    chromosomelist = iter(sorted(args.chrom_size.keys()))
    iterabull = iter(sorted(args.chrom_size.keys()))

    pool = Pool(max(args.procs - 1, 1), initializer=poolinit, initargs=(args,))

    cache = {}
    next_chrom = next(chromosomelist)
    for justdonechrom, smek1, smek2 in pool.imap_unordered(worker, iterabull, chunksize=1):
        if justdonechrom == next_chrom:
            # get results for chrom from shm and put in bw file
            shm_to_pybw_manager(justdonechrom, (smek1, smek2), bwl, args)
            try:
                next_chrom = next(chromosomelist)
                while next_chrom in cache:
                    # get results for chrom from shm and put in bw file
                    shm_to_pybw_manager(next_chrom, cache[next_chrom], bwl, args)
                    next_chrom = next(chromosomelist)

            except StopIteration:
                pass
        else:
            cache[justdonechrom] = smek1, smek2

    pool.close()
    pool.join()

    for bwfh in bwl:
        bwfh.close()


def main():
    """The main function.

    Handles arguments, then calls bam2bigwig with them.
    """
    parser = argparse.ArgumentParser(description="""
        Create a bigwig file with signal derived from a sorted and indexed bam file.  \
        Use the options to shape the signal according to what you are visualizing.    \
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\
        Reads can be filtered with -f and/or -F which work like in "samtools view".   \
        Try the option --explain_flags for more help with this.                       \
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\
        The reads can be shifted by --shift towards 5'->3' before being expanded by   \
        --extsize to expand their coverage. Try --coverage_examples for more help. You\
        can set strand specific shifting with --shift_p and --shift_n or use --atac   \
        to automatically correct the reads from ATACseq experiments.                  \
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\
        Use -p to parallelize (at the very least 2 cores are recommended).            \
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\
        Please report any issues/requests at github.com/PanosFirmpas/bamToBigWig      \
        """, add_help=False)
    req = parser.add_argument_group('rRequired arguments')

    req.add_argument('-b', '--bam', nargs='+', type=str, metavar='', help="""
                       The input .bam file. Must be sorted and indexed.
                       You can give more than one input files, separated by space, or even 
                       a wilcard expression, encased in quotation marks(e.g.,"./replicate*.bam"). """)

    req.add_argument('-chr', '--chrominfo', type=str, metavar='', help="""
                       A two column |<chromosome name> <total_length>| txt file.
                       The chromosome names must match those on the bam file. bamToBigWig only 
                       works on the chromosomes found in this file, so if you want to create a bigwig with only 
                       some chromosomes included, edit this input file accordingly.""")

    flt = parser.add_argument_group('Optional arguments - Filtering')

    flt.add_argument('-q', '--q', type=int, default=None, metavar='', help="""
                        Only include reads with mapping quality >= this option. Default is None, no q filter is applied.""")

    flt.add_argument('-f', '--filter_lc', type=int, default=2, metavar='', help="""
                       Only include reads with all bits of this argument set in their flag. Default is 2,
                       only reads mapped as "proper pair" are used. See --explain_flags""")

    flt.add_argument('-F', '--filter_uc', type=int, default=4, metavar='', help="""
                       Only include reads with none of the bits of this argument set in their flag. Default is 4, which 
                       filters out unmapped reads. If you don't want ANY such filter, you can set this option to 0 but this 
                       will likely cause the program to crash and burn since unmapped reads don't have some needed parameters like 
                       an alignment position. See --explain_flags for some more information.""")

    flt.add_argument('-tlu', '--t_len_upper', type=int, default=None, metavar='', help="""
                       Sets filtering so that only reads with absolute(tlen) lower than this are accepted.
                       This field is set by the mapper, but for paired-end sequencing it will typically refer to the
                       distance from the start of the + strand mapped read to the end of its - strand mapped mate. 
                       For ATAC experiments, setting this to approximately 120, will visualize
                       the "nucleosome free" reads. Default is "None", no fragment size filter is applied.""")

    flt.add_argument('-tll', '--t_len_lower', type=int, default=None, metavar='', help="""
                       Sets filtering so that only reads with absolute(tlen) greater than this are accepted.
                       See also --t_len_upper""")

    shi = parser.add_argument_group('Optional arguments - Shifting/Extending')

    shi.add_argument('-sh', '--shift', type=int, default=-35, metavar='', help="""
                        Arbitrary shift in bp. This value is used to move cutting ends (5') towards 5'->3'.
                        When this value is negative, ends will be moved toward 3'->5' direction.
                        If you want to extend the signal downstream of the original cutting end,
                        for example to visualize CHIPseq experiments, you will want to set this value to 0
                        and --extsize to how much you want to extend. If on the other hand you want to
                        extend the signal to both sides of the cutting end, for example if you are visualizing
                        ATACseq or DNAse-seq, you will want to set this to some negative value.
                        The default value -35 in combination with the default value of --extsize 70
                        will extend the signal by 35bp around the cutting end.
                        NOTE: this is different than the --atac option, or the --shift_p/shift_n options, and is applied on top of them.""")

    shi.add_argument('-exts', '--extsize', type=str, default='70', metavar='', help="""
                        Arbitrary extension size in bp. 
                        This value is used to extend each read towards the 3' end.  
                        Can be usefully combined with --shift. The default value is 70 
                        which in combination with the default --shift -35 extends the signal 
                        by 35bp upstream and downstream of the original cutting end.
                        If this is set to 'fragment', reads will be extended until their mate.""")

    shi.add_argument('-sh_p', '--shift_p', type=int, default=0, metavar='',
                     help="Like --shift (and applied on top of it), but only applied to reads that map on the positive strand.")

    shi.add_argument('-sh_n', '--shift_n', type=int, default=0, metavar='',
                     help="Like --shift (and applied on top of it), but only applied to reads that map on the negative strand.")

    shi.add_argument('-atac', '--atac', action='store_true', default=False, help="""
                        Shift reads mapping to the plus strand by +4 (5'->3') and reads that map
                        on the minus strand by +5 (5'-> 3'). This centers the cutting ends on the center of the
                        transposase 'event'. Overwrites --shift_p to 4 and --shift_n to 5""")

    opt = parser.add_argument_group('Optional arguments - Others')

    opt.add_argument('-p', '--procs', type=int, default=4, help="""
                       The number of processors to use. Defaults to 4. Workload is balanced by chromosomes.""")

    opt.add_argument('-o', '--out', type=str, metavar='', help="""
                        The output .bw file. If not given, it will be /path/to/input(.bam).bw""")

    opt.add_argument('-split', '--split_strands', action='store_true', default=False, help="""
                        Output separate files for + strand and - strand. They will automatically 
                        be named based on the output.bw (output_pstrand.bw / output_nstrand.bw)""")
    opt.add_argument('-vv', '--verbose', action='store_true', default=False,
                     help="""Prints out some helpful/debugging messages.""")

    opt.add_argument('-maxZ', '--maxzooms', type=int, default=10, metavar='', help=argparse.SUPPRESS)
    opt.add_argument('-shmb', '--shm_backend', type=str, default="shm://", metavar='', help=argparse.SUPPRESS)
    opt.add_argument('-n', '--dryrun', action='store_true', default=False, help=argparse.SUPPRESS)

    opt.add_argument('-h', '--help', action='store_true', default=False,
                     help=argparse.SUPPRESS)

    ht = parser.add_argument_group('Extra Help', "")

    ht.add_argument('-exfl', '--explain_flags', action='store_true',
                    help="Show some extra help about filtering reads with their flags.")
    # ht.add_argument('-exfi','--filter_examples', action='store_true',
    #                     help="Show some examples of filtering reads with their flags and exit.")
    ht.add_argument('-exco', '--coverage_examples', action='store_true',
                    help="Show some examples of filtering reads with their flags and exit.")

    # No arguments given
    if len(sys.argv[1:]) == 0:
        parser.print_help()
        parser.exit()

    # parse the arguments
    args = parser.parse_args()

    if args.help:
        parser.print_help()
        parser.exit()

    # Ok, we parsed our arguments, let's deal with them:
    if args.explain_flags:
        print(
            """
    Theory:
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\
    Sequencing reads in sam/bam files contain a binary flag, which is a       \n\
    combination of 12 bit-wise flags (i.e. 12 bits that can be 0 or 1).       \n\
    They are set by the aligner/mapper. This allows fast and efficient        \n\
    filtering of the reads.\n\n\
    If a read, for example, has its flag set to 2, we can see the bitwise flags\n\
    by converting 2 to its binary representation which is 000000000010 . The  \n\
    second bit/flag is set to 1, so it is True. This means that this imaginary\n\
    read belongs to a 'proper pair', meaning that both it and its mate were   \n\
    properly aligned by the aligner.\n\n\
    If you set the -f option to 2, you are making sure that only reads which  \n\
    have their second bitwise flag set to 1 will be used. By setting the -F   \n\
    option to 2, you are making sure that such reads are excluded.
    Some Examples:
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\
    # Only use 'proper pair' reads (usually implies more filters)\n\
    >> bamToBigWig -b in.bam -chr chrm.txt -f 2 \n\n\
    # EXCLUDE 'proper pair' reads \n\
    >> bamToBigWig -b in.bam -chr chrm.txt -F 2 \n\n\
    # Only use reads that mapped on the '-' strand:\n\
    >> bamToBigWig -b in.bam -chr chrm.txt -f 16 \n\n\
    # Only use 'proper pair' reads that mapped on the '-' strand:\n\
    >> bamToBigWig -b in.bam -chr chrm.txt -f 18 \n\n\
    # Only use reads that mapped on the '+' strand (we exclude the '-' reads):\n\
    >> bamToBigWig -b in.bam -chr chrm.txt -F 16 \n\n\
    # Only use 'proper pair' reads that mapped on the '+' strand:\n\
    >> bamToBigWig -b in.bam -chr chrm.txt -f 2 -F 16 \n\n\
    More:
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\
    An amazing resource to help you set your filters can be found here:  \n\
    https://broadinstitute.github.io/picard/explain-flags.html                \n\
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\
    """)
        parser.exit()

    if args.coverage_examples:
        print(
            """
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\
    # To visualize an ATACseq experiment, we use -atac to shift the reads \n\
    # appropriatelly, the --shift the reads upstream by 10 and expand the \n\
    # signal downstream of the cutting-site by 21. This expands the signal by \n\
    # 10bp each side of the cutting site.
    >> bamToBigWig -b in.bam -chr chrm.txt --shift -10 --extsize 21 \n\n\
    # To visualize a CHIPseq experiment, we expand the signal dowbstream of \n\
    # the cutting site by the experiment's fragment size, in this example: 300\n\
    >> bamToBigWig -b in.bam -chr chrm.txt --extsize 300 \n\n\
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\
    """)
        parser.exit()

    # chrominfo
    assert args.chrominfo is not None, "You need to specify an chrom_info file !"
    assert os.path.isfile(args.chrominfo), "Could not find {}".format(args.chrominfo)
    # get the chromosomes sorted by size, from the chrominfo file
    try:
        loc = []
        with open(args.chrominfo, "r") as fi:
            for line in fi:
                c, v = line.rstrip().split("\t")[:2]
                loc.append((c, int(v)))
        args.chrom_size = dict(loc)
    except:
        sys.exit("Couldn't properly parse the chromInfo file.")

    # bam input
    assert args.bam is not None, "You need to specify an input bam file !"
    args.bam = [x for y in args.bam for x in glob(os.path.expandvars(os.path.expanduser(y)))]
    for bamfp in args.bam:
        accepted, err = accept_as_input(bamfp)
        if not accepted:
            sys.exit(err)

    # handle output path
    if not args.out:
        if len(args.bam) > 1:
            sys.exit("""\n\n    You have specified more than one input files but no output name (the -o option ).
    Because of this I cannot decide what to call the output, please specify a name for it with -o.\n\n""")
        else:
            args.out = args.bam[0][:-4] + ".bw"
            # vrb
            print("Set output name to ", args.out)

    args.pathto, args.name = os.path.split(args.out)
    if args.pathto == "": args.pathto = "."
    assert os.path.exists(args.pathto), "This path [{}] doesn't exist".format(args.pathto)
    if args.name.endswith(".bw"):
        args.name = args.name[:-3]

    bwpl = []
    # split strands
    if args.split_strands:
        args.bw_p_path = os.path.join(args.pathto, args.name + "_pstrand.bw")
        args.bw_n_path = os.path.join(args.pathto, args.name + "_nstrand.bw")
        assert ~os.path.isfile(args.bw_p_path), "Output file [{}] already exists.".format(args.bw_p_path)
        assert ~os.path.isfile(args.bw_n_path), "Output file [{}] already exists.".format(args.bw_n_path)
        bwpl.append(args.bw_p_path)
        bwpl.append(args.bw_n_path)
    else:
        args.bw_path = os.path.join(args.pathto, args.name + ".bw")
        assert ~os.path.isfile(args.bw_path), "Output file [{}] already exists.".format(args.bw_path)
        bwpl.append(args.bw_path)

    # extsize
    if "fragment" == args.extsize:
        if args.split_strands:
            print("""\n\n   You have set both "fragment" type extension and "split_strands". \n\n""")
            sys.exit()
        pass
    else:
        try:
            args.extsize = int(args.extsize)
        except:
            sys.exit("extsize needs to be >= 0 ")
        assert args.extsize >= 0, "extsize needs to be >= 0 "

    # shifting
    if args.atac:
        if (args.shift_p != 0) or (args.shift_n != 0):
            print("""\n\n   You seem to have set both a specific value for --shift_p or --shift_n
   AND the -atac flag. The atac flag overrides the other two values so please
   use either shift_n/shift_p OR -atac/ \n\n""")
            sys.exit()

        args.shift_p = 4
        args.shift_n = 5
    # this used to default to None so the code was built around that
    # Now that it sensibly defaults to 4, the user would set it to 0
    # to get the same behaviour as None
    # so instead of changing the code ill just change this here
    if args.filter_uc == 0:
        args.filter_uc = None

    # verbose print
    args.vprint = print if args.verbose else lambda *a, **k: None

    args.vprint("Arguments: ", file=sys.stdout)
    args.vprint("", file=sys.stdout)
    args.vprint("chromInfo file: \t {}".format(args.chrominfo), file=sys.stdout)
    args.vprint("ATAC mode: \t {}".format("True" if args.atac else "False"),
                file=sys.stdout)
    args.vprint(("Split strands: \t {}"
                 .format("True" if args.split_strands else "False")), file=sys.stdout)
    args.vprint("Processors to use: \t {}".format(args.procs), file=sys.stdout)
    args.vprint("BigWig maxZooms: \t {}".format(args.maxzooms), file=sys.stdout)
    args.vprint("", file=sys.stdout)
    # args.vprint("{} ".format(args.shm_backend), file=sys.stdout)
    args.vprint("Mapping Quality lower limit: \t {}".format(args.q),
                file=sys.stdout)
    args.vprint("Fragment Length uper limit: \t {}".format(args.t_len_upper),
                file=sys.stdout)
    args.vprint("Fragment Length lower limit: \t {}".format(args.t_len_lower),
                file=sys.stdout)
    args.vprint("-f flag: \t {}".format(args.filter_lc), file=sys.stdout)
    args.vprint("-F flag: \t {}".format(args.filter_uc), file=sys.stdout)
    args.vprint("", file=sys.stdout)
    args.vprint("Extend by: \t {}".format(args.extsize), file=sys.stdout)
    args.vprint("Shift all reads by: \t {}".format(args.shift), file=sys.stdout)
    args.vprint("Shift + reads by: \t {}".format(args.shift_p), file=sys.stdout)
    args.vprint("Shift - reads by: \t {}".format(args.shift_n), file=sys.stdout)
    args.vprint("", file=sys.stdout)

    args.vprint("", file=sys.stdout)
    for enum, fp in enumerate(args.bam):
        args.vprint("Bam input file[{}]: {} ".format(enum, fp), file=sys.stdout)
    for enum, fp in enumerate(bwpl):
        args.vprint("Output file[{}]: {} ".format(enum, fp), file=sys.stdout)

    if not args.dryrun:
        bamtobw(args)


if __name__ == "__main__":
    main()



from api.xuyuxing.genome.repeatmasker import repeatmasker_parser
from lib.common.genome.seq_base import read_fasta_by_faidx, sequence_entropy
from lib.common.math.interval import merge_intervals, interval_minus_set
import os
import re
import random
from lib.common.os import mkdir, multiprocess_running
from lib.xuyuxing.math.stats import get_threshold
from lib.common.math.bin_and_window import split_sequence_to_bins

def format_hgt_sequence_database(prefix, genome_file, repeat_lib, repeatmasker_out, bin_length, entropy_threshold=3, min_fragment_length=50):
    # read
    repeatmasker_info = repeatmasker_parser(repeatmasker_out)
    repeat_unit = read_fasta_by_faidx(repeat_lib)
    repeat_unit = {i.split("#")[0]:str(repeat_unit[i].seq) for i in repeat_unit}
    genome_seq = read_fasta_by_faidx(genome_file)
    genome_seq = {i: str(genome_seq[i].seq) for i in genome_seq}

    # get unmasked range
    pass_repeat_feature = ['Simple_repeat', 'Low_complexity', 'rRNA']

    genome_range = {}
    for i in genome_seq:
        genome_range[i] = (1,len(genome_seq[i]))

    masked_range = {}
    for i in genome_seq:
        masked_range[i] = []

    repeat_seq_length = 0
    used_repeat_id = []
    for feature_type in repeatmasker_info:
        for repeat_id in repeatmasker_info[feature_type]:
            if not repeat_id in repeat_unit and feature_type not in pass_repeat_feature:
                raise ValueError("repeatmasker out file have repeat seq not in repeat lib fasta!")
            
            if feature_type not in pass_repeat_feature:
                used_repeat_id.append(repeat_id)
                repeat_seq_length += len(repeat_unit[repeat_id])

            for case in repeatmasker_info[feature_type][repeat_id].case_list:
                masked_range[case.q_name].append((case.q_start, case.q_end))

    for i in genome_seq:
        masked_range[i] = merge_intervals(masked_range[i])

    unmasked_range = {}
    for i in genome_seq:
        unmasked_range[i] = interval_minus_set(genome_range[i], masked_range[i])

    unmasked_length = 0
    for i in unmasked_range:
        for range_tmp in unmasked_range[i]:
            unmasked_length += (range_tmp[1] - range_tmp[0])

    genome_length = 0
    for i in genome_seq:
        genome_length += len(genome_seq[i])

    # output fasta and map file
    fragment_map = {}
    num = 0
    with open(prefix+".fragment.fasta", 'w') as f:

        for contig in unmasked_range:
            print(contig)
            for range_tmp in unmasked_range[contig]:
                for index, start, end in split_sequence_to_bins(range_tmp[1], start=range_tmp[0] ,bin_length=bin_length):
                    frag_id = "F" + str(num)
                    fragment_map[frag_id] = (contig, start, end, 'unmask')
                    
                    seq_in_range = genome_seq[contig][start - 1: end].upper()
                    
                    if len(seq_in_range) >= min_fragment_length and sequence_entropy(seq_in_range, 3) > entropy_threshold:
                        f.write(">%s\n%s\n" % (frag_id, seq_in_range))

                        num += 1

        for repeat_id in used_repeat_id:
            print(repeat_id)
            for index, start, end in split_sequence_to_bins(len(repeat_unit[repeat_id]), start=1 ,bin_length=bin_length):
                frag_id = "F" + str(num)
                fragment_map[frag_id] = (repeat_id, start, end, 'mask')

                seq_in_range = repeat_unit[repeat_id][start - 1: end].upper()

                if len(seq_in_range) >= min_fragment_length and sequence_entropy(seq_in_range, 3) > entropy_threshold:
                    f.write(">%s\n%s\n" % (frag_id, seq_in_range))

                    num += 1

    with open(prefix+".fragment.map", 'w') as f:
        for frag_id in fragment_map:
            contig, start, end, type_now = fragment_map[frag_id]
            f.write("%s\t%s\t%d\t%d\t%s\n" % (frag_id, contig, start, end, type_now))

    print(genome_length, unmasked_length, repeat_seq_length, (unmasked_length+repeat_seq_length)/genome_length)



if __name__ == "__main__":
    genome_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Cuscuta_australis/T267555N0.genome.fasta'
    repeat_lib = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Cuscuta_australis/T267555N0.repeat/allRepeats.clean.lib'
    repeatmasker_out = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Cuscuta_australis/RepeatMasker.out/T267555N0.genome.fasta.out'
    prefix = '/lustre/home/xuyuxing/Work/Big_HGT2/Cau'
    bin_length = 100
    entropy_threshold=3
    min_fragment_length=50

    format_hgt_sequence_database(prefix, genome_file, repeat_lib, repeatmasker_out, bin_length, entropy_threshold=3, min_fragment_length=50)


    genome_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phaseolus_vulgaris/T3885N0.genome.fasta'
    repeat_lib = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phaseolus_vulgaris/T3885N0.repeat/allRepeats.clean.lib'
    repeatmasker_out = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phaseolus_vulgaris/RepeatMasker.out/T3885N0.genome.fasta.out'
    prefix = '/lustre/home/xuyuxing/Work/Big_HGT2/Pvu'
    bin_length = 100
    entropy_threshold=3
    min_fragment_length=50

    format_hgt_sequence_database(prefix, genome_file, repeat_lib, repeatmasker_out, bin_length, entropy_threshold=3, min_fragment_length=50)

    genome_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Helianthus_annuus/T4232N0.genome.fasta'
    repeat_lib = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Helianthus_annuus/T4232N0.repeat/allRepeats.clean.lib'
    repeatmasker_out = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Helianthus_annuus/RepeatMasker.out/T4232N0.genome.fasta.out'
    prefix = '/lustre/home/xuyuxing/Work/Big_HGT2/Han'
    bin_length = 100
    entropy_threshold=3
    min_fragment_length=50

    format_hgt_sequence_database(prefix, genome_file, repeat_lib, repeatmasker_out, bin_length, entropy_threshold=3, min_fragment_length=50)

    genome_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Ipomoea_nil/T35883N0.genome.fasta'
    repeat_lib = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Ipomoea_nil/T35883N0.repeat/allRepeats.clean.lib'
    repeatmasker_out = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Ipomoea_nil/RepeatMasker.out/T35883N0.genome.fasta.out'
    prefix = '/lustre/home/xuyuxing/Work/Big_HGT2/Ini'
    bin_length = 100
    entropy_threshold=3
    min_fragment_length=50

    format_hgt_sequence_database(prefix, genome_file, repeat_lib, repeatmasker_out, bin_length, entropy_threshold=3, min_fragment_length=50)

    genome_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/T91201N0.genome.fasta'
    repeat_lib = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.repeat/allRepeats.clean.lib'
    repeatmasker_out = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/RepeatMasker.out/Gel.genome.v2.0.fasta.out'
    prefix = '/lustre/home/xuyuxing/Work/Big_HGT2/Gel'
    bin_length = 100
    entropy_threshold=3
    min_fragment_length=50

    format_hgt_sequence_database(prefix, genome_file, repeat_lib, repeatmasker_out, bin_length, entropy_threshold=3, min_fragment_length=50)


import os
import pickle
from Bio import Phylo
from itertools import combinations
from config import project_dir
from lib.common.util import pickle_step
from lib.common.os import have_file, logging_init, mkdir, cmd_run, multiprocess_running
from lib.common.evolution.orthotools2 import read_species_info, OrthoGroups
from lib.common.evolution.tree_operate import lookup_by_names, get_leaves, get_MRCA
from api.xuyuxing.comp_genome.phylogenomics import prepare_OG_work_dir, tree_pipeline, get_orthogroups_pipeline
from api.xuyuxing.comp_genome.mcscan import WGD_check_pipeline
from collections import Counter
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patheffects as path_effects
import matplotlib.ticker as ticker


def write_keep_gene_id_file(filter_work_dir, sp_list, keep_OG_list, OG_dict):
    # open
    keep_file_handle_dict = {}
    keep_file_dict = {}
    for sp_id in sp_list:
        keep_file_dict[sp_id] = "%s/%s.keep.id" % (filter_work_dir, sp_id)
        keep_file_handle_dict[sp_id] = open(keep_file_dict[sp_id], 'w')

    # write
    keep_gene_num_dict = {}
    for sp_id in sp_list:
        keep_gene_num_dict[sp_id] = 0
        for OG_id in keep_OG_list:
            og = OG_dict[OG_id]
            for g in og.gene_dict[sp_id]:
                g_id = g.id
                keep_file_handle_dict[sp_id].write(g_id+"\n")
                keep_gene_num_dict[sp_id] += 1

    # close
    for sp_id in sp_list:
        keep_file_handle_dict[sp_id].close()

    return keep_file_dict


def make_trees(OG_tsv_file, sp_info_dict, sp_tree_file, top_dir, method, threads):
    mkdir(top_dir, True)

    # print("read OG file")
    OGs = OrthoGroups(OG_tsv_file=OG_tsv_file)

    og_list = list(OGs.OG_dict.keys())

    for og_id in og_list:
        og = OGs.OG_dict[og_id]
        if len(og.gene_list) <= 3 or len([og.species_stat[i] for i in og.species_stat if og.species_stat[i] > 0]) <= 1:
            del OGs.OG_dict[og_id]
            OGs.OG_id_list.remove(og_id)

    # print("prepare_OG_work_dir")
    OGs = prepare_OG_work_dir(
        OGs, sp_info_dict, top_dir, sp_tree_file, need_cds_flag=True)

    # print("treeing")
    OGs = tree_pipeline(OGs, method, threads=threads)

    return OGs


def save_WGD_check_pipeline(save_file, *args):
    q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict = WGD_check_pipeline(
        *args)
    OUT = open(save_file, 'wb')
    pickle.dump((q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict,
                 s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict), OUT)
    OUT.close()


def get_all_orthogroups(raw_OGs, OG_dir, node_list):
    mkdir(OG_dir, True)

    OG_dict = {}
    OG_tsv_dict = {}

    for taxon in node_list:
        # print(taxon)
        OG_dict[taxon] = get_orthogroups_pipeline(
            raw_OGs, taxonomy_level=taxon, conserved_arguments=None, threads=56)
        OG_tsv_file = OG_dir+"/"+taxon+".tsv"
        OG_dict[taxon].write_OG_tsv_file(OG_tsv_file)
        OG_tsv_dict[taxon] = OG_tsv_file

    return OG_dict, OG_tsv_dict


def get_gene_cover(sp_pair_list, OCF_pickle_dict):

    arrays = [['sp_pair_info'] * 5 + ['count'] * 11 + ['no_zero_ratio'] * 10,
              ['query_species', 'subject_species', 'subject_gene_num', 'no_zero_gene', 'no_zero_ratio'] + list(range(11)) + list(range(1, 11))]

    tuples = list(zip(*arrays))

    index = pd.MultiIndex.from_tuples(tuples)

    gene_cover_df = pd.DataFrame(columns=index)

    num = 0
    for sp_pair in sp_pair_list:
        # print(sp_pair)
        ocf_pyb_file = OCF_pickle_dict[sp_pair]

        TEMP = open(ocf_pyb_file, 'rb')
        q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict = pickle.load(
            TEMP)
        TEMP.close()

        q_count_dict = dict(Counter([q_gene_covered_dict[i]
                                     for i in q_gene_covered_dict]))
        q_num = list(range(max(q_count_dict.keys())+1))
        q_count = [q_count_dict[i] if i in q_count_dict else 0 for i in q_num]
        q_zero = q_count_dict[0]
        q_sum = sum(q_count)
        q_no_zero_sum = q_sum - q_zero
        q_no_zero_sum_ratio = q_no_zero_sum/q_sum
        q_no_zero_num = q_num[1:]
        q_no_zero_count = [q_count_dict[i]
                           if i in q_count_dict else 0 for i in q_no_zero_num]
        q_no_zero_ratio = [i/q_no_zero_sum for i in q_no_zero_count]

        s_count_dict = dict(Counter([s_gene_covered_dict[i]
                                     for i in s_gene_covered_dict]))
        s_num = list(range(max(s_count_dict.keys())+1))
        s_count = [s_count_dict[i] if i in s_count_dict else 0 for i in s_num]
        s_zero = s_count_dict[0]
        s_sum = sum(s_count)
        s_no_zero_sum = s_sum - s_zero
        s_no_zero_sum_ratio = s_no_zero_sum/s_sum
        s_no_zero_num = s_num[1:]
        s_no_zero_count = [s_count_dict[i]
                           if i in s_count_dict else 0 for i in s_no_zero_num]
        s_no_zero_ratio = [i/s_no_zero_sum for i in s_no_zero_count]

    #     print(sp_pair)
    #     print(q_no_zero_sum_ratio)
    #     print(q_no_zero_ratio)
    #     print(s_no_zero_sum_ratio)
    #     print(s_no_zero_ratio)

#         pair_dict[sp_pair]['gene_cover_stat'] = (q_no_zero_sum_ratio, q_no_zero_ratio, s_no_zero_sum_ratio, s_no_zero_ratio)

        i, j = sp_pair

        gene_cover_df.loc[num] = [i, j, sum(s_count), s_no_zero_sum, s_no_zero_sum_ratio] + s_count + [
            0] * (11 - len(s_count)) + s_no_zero_ratio + [0] * (11 - len(s_count))
        num += 1
        gene_cover_df.loc[num] = [j, i, sum(q_count), q_no_zero_sum, q_no_zero_sum_ratio] + q_count + [
            0] * (11 - len(q_count)) + q_no_zero_ratio + [0] * (11 - len(q_count))
        num += 1

    return gene_cover_df


def hWGDdetector_main(args):
    args.work_dir = os.path.abspath(args.work_dir)

    mkdir(args.work_dir, True)

    logger = logging_init("hWGDdetector", args.log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    # step1
    logger.info("Step1: load species info")
    sp_info_dict = read_species_info(args.sp_info_excel)
    sp_tree = Phylo.read(args.sp_tree_file, "newick")
    sp_tree_dict = lookup_by_names(sp_tree)
    sp_list = get_leaves(sp_tree, return_name=True)
    logger.info("    There are %d species in the dataset" % (len(sp_list)))

    # step2
    logger.info("Step2: load and filter orthogroups")
    OGs = OrthoGroups(OG_tsv_file=args.OG_tsv_file)

    huge_OG_list = []
    for i in OGs.OG_dict:
        og = OGs.OG_dict[i]
        for j in og.gene_dict:
            if len(og.gene_dict[j]) > args.huge_OG_gene_num:
                huge_OG_list.append(i)

    huge_OG_list = list(set(huge_OG_list))
    keep_OG_list = sorted(
        list(set(list(OGs.OG_dict.keys())) - set(list(huge_OG_list))))
    logger.info("    There are %d OGs in the database, %d of which are huge OGs, and we keep the remaining %d OGs" % (
        len(OGs.OG_dict), len(huge_OG_list), len(keep_OG_list)))

    logger.info("    Filtering cds, pt, gff files")
    filter_work_dir = args.work_dir + "/gene_filter"
    mkdir(filter_work_dir, True)
    keep_file_dict = write_keep_gene_id_file(
        filter_work_dir, sp_list, keep_OG_list, OGs.OG_dict)

    for sp_id in sp_list:
        sp = sp_info_dict[sp_id]

        # aa
        input_file = sp.pt_file
        output_file = "%s/%s.aa.filter.fasta" % (filter_work_dir, sp_id)
        gene_id_list = keep_file_dict[sp_id]

        cmd_string = "python %s/SeqParser.py FastaByID -f %s -o %s %s" % (
            project_dir, gene_id_list, output_file, input_file)
        if not have_file(output_file):
            cmd_run(cmd_string, silence=True)
        sp.raw_pt_file = sp.pt_file
        sp.pt_file = output_file
        # logger.info("%s aa ok" % sp_id)

        # cds
        input_file = sp.cds_file
        output_file = "%s/%s.cds.filter.fasta" % (filter_work_dir, sp_id)
        gene_id_list = keep_file_dict[sp_id]

        cmd_string = "python %s/SeqParser.py FastaByID -f %s -o %s %s" % (
            project_dir, gene_id_list, output_file, input_file)
        if not have_file(output_file):
            cmd_run(cmd_string, silence=True)
        sp.raw_cds_file = sp.cds_file
        sp.cds_file = output_file
        # logger.info("%s cds ok" % sp_id)

        # gff
        input_file = sp.gff_file
        output_file = "%s/%s.filter.gff3" % (filter_work_dir, sp_id)
        gene_id_list = keep_file_dict[sp_id]

        cmd_string = "python %s/GenFeatTools.py ExtractGFF %s %s %s" % (
            project_dir, gene_id_list, input_file, output_file)
        if not have_file(output_file):
            cmd_run(cmd_string, silence=True)
        sp.raw_gff_file = sp.gff_file
        sp.gff_file = output_file
        # logger.info("%s gff ok" % sp_id)

    # step3
    logger.info("Step3: Identification of orthologous genes")
    phylo_work_dir = args.work_dir + "/phylogenomics"
    mkdir(phylo_work_dir, True)

    keepOGs = OrthoGroups(from_OG_list=[
                          OGs.OG_dict[i] for i in keep_OG_list], species_list=OGs.species_list)
    keepOGs_file = phylo_work_dir + "/filter.orthogroups.tsv"
    keepOGs.write_OG_tsv_file(keepOGs_file)

    tree_dir = phylo_work_dir + "/tree"
    mkdir(tree_dir, True)

    logger.info("    Building trees")
    # fasttree
    fasttree_OGs_pyb = tree_dir + "/fasttree_OGs.pyb"
    FT_OGs = pickle_step(make_trees, [
                         keepOGs_file, sp_info_dict, args.sp_tree_file, tree_dir+"/fasttree", "fasttree", args.threads], fasttree_OGs_pyb)

    # # treebest
    # treebest_OGs_pyb = tree_dir + "/treebest_OGs.pyb"
    # TB_OGs = pickle_step(make_trees, [
    #                      OG_tsv_file, sp_info_dict, sp_tree_file, tree_dir+"/treebest", "treebest"], treebest_OGs_pyb)

    # # raxml
    # raxml_OGs_pyb = tree_dir + "/raxml_OGs.pyb"
    # RM_OGs = pickle_step(make_trees, [OG_tsv_file, sp_info_dict, species_tree_file, tree_dir+"/raxml", "raxml"], raxml_OGs_pyb)

    logger.info("    Get orthologous genes")
    node_list = [clade.name for clade in sp_tree.get_nonterminals()]
    logger.info("        There are %d nodes in species tree" %
                (len(node_list)))

    OG_dir = args.work_dir + "/orthogroups"

    subOGs_dict_pyb = OG_dir + "/subOGs_dict.pyb"
    subOGs_dict, subOGs_tsv_dict = pickle_step(
        get_all_orthogroups, [FT_OGs, OG_dir, node_list], subOGs_dict_pyb)

    species_tree_clade_dict = lookup_by_names(sp_tree)

    mrca_dict = {}
    for sp_pair in combinations(sp_list, 2):
        mrca_dict[tuple(sorted(sp_pair))] = get_MRCA(
            species_tree_clade_dict[sp_pair[0]], species_tree_clade_dict[sp_pair[1]], sp_tree).name

    sp_pair_list = list(mrca_dict.keys())

    logger.info("        There are %d species, %d species-pair, all orthologous gene extracted!" %
                (len(sp_list), len(sp_pair_list)))

    # step4
    logger.info("Step4: Use orthologous gene build OCF")
    logger.info("    Running mcscanxh")
    mcscanxh_dir = args.work_dir + "/mcscanxh"
    mkdir(mcscanxh_dir, True)

    mcscanxh_file_dict = {}

    args_list = []

    for sp_pair in sp_pair_list:
        i, j = sp_pair
        mrca = mrca_dict[sp_pair]
        orthologous_file = subOGs_tsv_dict[mrca]
        mcscanxh_file_dict[sp_pair] = "%s/%s_vs_%s_h" % (
            mcscanxh_dir, i, j)
        cmd_string = "python %s/OrthoTools.py mcscanxhGO -sp1 %s -sp2 %s -sp2_gff %s -w %s %s %s" % (
            project_dir, i, j, sp_info_dict[j].gff_file, mcscanxh_file_dict[sp_pair], sp_info_dict[i].gff_file, orthologous_file)

        args_list.append((cmd_string, mcscanxh_dir, 1, True))

    multiprocess_running(cmd_run, args_list, 16, silence=False)

    logger.info("    Load OCFs")

    OCF_pickle_dict = {}
    for sp_pair in sp_pair_list:
        # print(sp_pair)

        i, j = sp_pair

        mcscanx_collinearity_file = mcscanxh_file_dict[sp_pair] + \
            "/mcscanx.collinearity"
        query_gff3 = sp_info_dict[i].gff_file
        subject_gff3 = sp_info_dict[j].gff_file

        ocf_pyb_file = mcscanxh_file_dict[sp_pair] + "/ocf.pyb"
        OCF_pickle_dict[sp_pair] = ocf_pyb_file

        if os.path.exists(ocf_pyb_file):
            continue

        save_WGD_check_pipeline(
            ocf_pyb_file, mcscanx_collinearity_file, i, query_gff3, None, j, subject_gff3, None)

    # step5
    logger.info("Step5: Identification of hWGD")

    gene_cover_pyb = args.work_dir + "/gene_cover_df.pyb"
    gene_cover_df = pickle_step(
        get_gene_cover, [sp_pair_list, OCF_pickle_dict], gene_cover_pyb)

    gene_cover_excel = args.work_dir + "/gene_cover_df.xlsx"
    writer = pd.ExcelWriter(gene_cover_excel)
    gene_cover_df.to_excel(writer)
    writer.save()

    fig = plt.figure(figsize=(20, 20))

    speci_num = len(sp_list)

    num = 1
    for i in sp_list:
        for j in sp_list:

            ax = fig.add_subplot(speci_num, speci_num, num)
            ax.xaxis.set_major_locator(ticker.MultipleLocator(base=1))
            ax.set_xlim(0, 8)
            ax.set_ylim(0, 1)

            if not num % speci_num == 1 and num != 2:
                labels = [item.get_text() for item in ax.get_yticklabels()]

                empty_string_labels = ['']*len(labels)
                ax.set_yticklabels(empty_string_labels)

            if num < speci_num * (speci_num - 1):
                labels = [item.get_text() for item in ax.get_xticklabels()]

                empty_string_labels = ['']*len(labels)
                ax.set_xticklabels(empty_string_labels)

            if i == j:
                text = ax.text(3.5, 0.5, i, ha='center', va='center', size=20)
                text.set_path_effects([path_effects.Normal()])
                ax.spines['top'].set_visible(False)
                ax.spines['right'].set_visible(False)
                ax.spines['left'].set_visible(False)
                ax.spines['bottom'].set_visible(False)
                ax.get_xaxis().set_visible(False)
                ax.get_yaxis().set_visible(False)
                num += 1
                continue
            else:
                row = gene_cover_df[(gene_cover_df.sp_pair_info.query_species == i) & (
                    gene_cover_df.sp_pair_info.subject_species == j)]
                plot_no_zero_sum_ratio = float(row.sp_pair_info.no_zero_ratio)
                plot_no_zero_ratio = list(row.no_zero_ratio.iloc[0])[0:8]

    #             sp_pair = (i,j)
    #             sp_pair_r = (j,i)
    #             if sp_pair in pair_dict:
    #                 q_no_zero_sum_ratio, q_no_zero_ratio, s_no_zero_sum_ratio, s_no_zero_ratio = pair_dict[sp_pair]['gene_cover_stat']
    #                 plot_no_zero_sum_ratio, plot_no_zero_ratio = s_no_zero_sum_ratio, s_no_zero_ratio
    #             else:
    #                 q_no_zero_sum_ratio, q_no_zero_ratio, s_no_zero_sum_ratio, s_no_zero_ratio = pair_dict[sp_pair_r]['gene_cover_stat']
    #                 plot_no_zero_sum_ratio, plot_no_zero_ratio = q_no_zero_sum_ratio, q_no_zero_ratio

                text = ax.text(3.5, 0.80, "%.2f%%" %
                               (plot_no_zero_sum_ratio*100), size=12)
                ax.bar(list(range(1, len(plot_no_zero_ratio)+1)),
                       plot_no_zero_ratio)
                num += 1

    plt.show()

    save_file = args.work_dir + "/hWGD_fig.pdf"

    fig.savefig(save_file, format='pdf', facecolor='none',
                edgecolor='none', bbox_inches='tight')


if __name__ == "__main__":
    class abc():
        pass

    args = abc()

    args.work_dir = '/lustre/home/xuyuxing/Work/Orobanchaceae/publish2/2.WGD/tmp'
    args.log_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/publish2/2.WGD/tmp/log'
    args.sp_info_excel = '/lustre/home/xuyuxing/Work/Orobanchaceae/publish2/2.WGD/tmp/Oro_ref.xlsx'
    args.sp_tree_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/publish2/2.WGD/tmp/species.tre'
    args.OG_tsv_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/publish2/2.WGD/tmp/Orthogroups.tsv'
    args.huge_OG_gene_num = 30
    args.threads = 56

    hWGDdetector_main(args)



import os
from collections import OrderedDict
from api.xuyuxing.resource.ncbi_genome import build_ncbi_genome_database, md5checksums_checker, ascp_download, wget_download, genome_chooser
from lib.common.fileIO import read_list_file
import re
from lib.xuyuxing.evolution.taxonomy import build_taxon_database, read_tax_record_dict_db, store_tax_record_into_sqlite
from lib.xuyuxing.base.common_command import log_print
from api.xuyuxing.file_parser.fileIO import tsv_file_dict_parse, tsv_file_parse
from lib.common.os import md5sum_check
import time
from config import taxon_dump
import sys
from ftplib import FTP
from retry import retry


def ncbi_genome_downloader(args):

    if args.download_way == 'ascp':
        download_function = ascp_download
    elif args.download_way == 'wget':
        download_function = wget_download

    if args.assembly_summary_refseq_file is None:
        args.assembly_summary_refseq_file = args.ncbi_genome_dir + \
            "/assembly_summary_refseq.txt"
        args.assembly_summary_genbank_file = args.ncbi_genome_dir + \
            "/assembly_summary_genbank.txt"

    # download list
    log_print("Read download accession list")
    acc_list = read_list_file(args.list_of_accession_ID)

    if args.download_type:
        download_type = args.download_type.split(",")
    else:
        download_type = ["gff_file", "pt_file", "cds_file", "genome_file"]

    log_print("There are %d accesion will be download, will download their: %s" % (
        len(acc_list), ", ".join(download_type)))

    # parse metadata
    log_print("Read metadata from NCBI")
    record_dict = build_ncbi_genome_database(
        args.assembly_summary_genbank_file, args.assembly_summary_refseq_file)
    record_dict = {i: record_dict[i] for i in acc_list}

    # prepare
    log_print("Prepare download information")
    data_dir = args.ncbi_genome_dir
    for acc_id in record_dict:
        record_tmp = record_dict[acc_id]
        ftp_path = record_tmp.ftp_path
        ftp_dir_path = ftp_path.replace("ftp://ftp.ncbi.nlm.nih.gov", "")
        data_dir_tmp = data_dir + "/" + record_tmp.assembly_accession
        file_path = {
            "gff_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_genomic.gff.gz",
                         data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_genomic.gff.gz"),
            "pt_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_translated_cds.faa.gz",
                        data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_translated_cds.faa.gz"),
            "cds_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_cds_from_genomic.fna.gz",
                         data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_cds_from_genomic.fna.gz"),
            "genome_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_genomic.fna.gz",
                            data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_genomic.fna.gz"),
            "md5_file": (ftp_dir_path + "/" + "md5checksums.txt", data_dir_tmp + "/" + "md5checksums.txt")
        }
        para_dict_tmp = {
            "speci_accession": re.search("(^\S+\_\d+)\.\d+", record_tmp.assembly_accession).group(1),
            "assembly_accession": record_tmp.assembly_accession,
            "full_accession": ftp_path.split("/")[-1],
            "files": file_path,
            "ftp_dir_path": ftp_dir_path,
            "data_dir_tmp": data_dir_tmp
        }
        record_tmp.para_dict = para_dict_tmp

    # md5file
    log_print("Check NCBI md5checksums")

    if args.renew_md5_flag is True:
        log_print("\t\tRenew all md5checksums")
    else:
        log_print("\t\tCheck missing md5checksums")

    for acc_id in record_dict:
        record_tmp = record_dict[acc_id]
        md5checksums_checker(record_tmp.para_dict,
                             args.renew_md5_flag, download_way=args.download_way)

    log_print("Add md5 string to file")

    for acc_id in record_dict:
        para_dict_tmp = record_dict[acc_id].para_dict
        md5_file = para_dict_tmp["files"]["md5_file"][1]
        md5_list = tsv_file_parse(md5_file, seq=r' ')
        for i in para_dict_tmp["files"]:
            for j in md5_list:
                md5_string, nothing, file_name = md5_list[j]
                if para_dict_tmp["files"][i][0].split("/")[-1] == file_name.split("/")[-1]:
                    record_dict[acc_id].para_dict["files"][i] = tuple(
                        list(record_dict[acc_id].para_dict["files"][i]) + [md5_string])

    # check if downloaded
    log_print("Check the completed download file")
    start_time = time.time()
    num = 0
    ok_file_num = 0
    renew_file_num = 0
    miss_file_num = 0
    for acc_id in record_dict:
        num = num + 1
        para_dict_tmp = record_dict[acc_id].para_dict
        for file_type in para_dict_tmp["files"]:
            if len(para_dict_tmp["files"][file_type]) == 3:
                online_path, file_path, md5_string = para_dict_tmp["files"][file_type]
                if md5sum_check(file_path, md5_string):
                    log_tmp = "ok"
                    if file_type in download_type:
                        ok_file_num += 1
                else:
                    log_tmp = "renew"
                    if file_type in download_type:
                        renew_file_num += 1
                record_dict[acc_id].para_dict["files"][file_type] = tuple(
                    list(record_dict[acc_id].para_dict["files"][file_type]) + [log_tmp])
            else:
                log_tmp = "miss"
                if file_type in download_type:
                    miss_file_num += 1
                record_dict[acc_id].para_dict["files"][file_type] = tuple(
                    list(record_dict[acc_id].para_dict["files"][file_type]) + ["", log_tmp])
        round_time = time.time()
        if round_time - start_time > 10:
            log_print("\t\t%d/%d" % (num, len(record_dict)))
            start_time = round_time

    log_print("Total: %d, %d is downloaded, %d need download now, %d are missed in NCBI" % (
        ok_file_num+renew_file_num+miss_file_num, ok_file_num, renew_file_num, miss_file_num))

    # Download uncompleted file
    log_print("Download uncompleted file")

    for acc_id in record_dict:
        for i in record_dict[acc_id].para_dict["files"]:
            if i in download_type:
                url_path, file_path, md5_string, log_tmp = record_dict[acc_id].para_dict["files"][i]
                if log_tmp == "renew":
                    if download_function(url_path, file_path):
                        record_dict[acc_id].para_dict["files"][i] = list(
                            record_dict[acc_id].para_dict["files"][i][:3]) + ["ok"]
                    else:
                        raise EnvironmentError("Check network!")

    # make report
    col_name_list = ["assembly_accession", "species_taxid", "kingdom", "order", "family", "genus", "organism_name", "refseq_category", "group", "gff_file", "gff_file_md5",
                     "gff_file_status", "pt_file", "pt_file_md5", "pt_file_status", "cds_file", "cds_file_md5", "cds_file_status", "genome_file", "genome_file_md5", "genome_file_status", "ftp_path"]

    report_file = args.ncbi_genome_dir + "/downloaded_genome.txt"

    output_dict = OrderedDict()
    if os.path.exists(report_file):
        info_dict = tsv_file_dict_parse(report_file, key_col="assembly_accession")
        for i in info_dict:
            output_dict[i] = [info_dict[i][j] for j in col_name_list]

    if args.taxonomy_dir:
        tax_record_dict = build_taxon_database(args.taxonomy_dir)
    else:
        tax_record_dict = read_tax_record_dict_db(taxon_dump+"/taxdump_xyx.db")

    for acc_id in record_dict:
        record_tmp = record_dict[acc_id]

        taxon_info_list = [record_tmp.species_taxid, "None",
                           "None", "None", "None", record_tmp.organism_name]
        if record_tmp.species_taxid in tax_record_dict:
            taxon_tmp = tax_record_dict[record_tmp.species_taxid]
            kingdom = tax_record_dict[taxon_tmp.kingdom].sci_name if hasattr(
                taxon_tmp, 'kingdom') else "None"
            order = tax_record_dict[taxon_tmp.order].sci_name if hasattr(
                taxon_tmp, 'order') else "None"
            family = tax_record_dict[taxon_tmp.family].sci_name if hasattr(
                taxon_tmp, 'family') else "None"
            genus = tax_record_dict[taxon_tmp.genus].sci_name if hasattr(
                taxon_tmp, 'genus') else "None"
            taxon_info_list = [record_tmp.species_taxid, kingdom,
                               order, family, genus, record_tmp.organism_name]

        file_info_list = []
        for file_type in ["gff_file", "pt_file", "cds_file", "genome_file"]:
            url_str, file_path, md5_string, status = record_tmp.para_dict['files'][file_type]
            file_info_list.extend([file_path, md5_string, status])

        output_dict[acc_id] = [record_tmp.assembly_accession] + \
            taxon_info_list + file_info_list + [record_tmp.ftp_path]

    with open(args.ncbi_genome_dir + "/downloaded_genome.txt", 'w') as f:
        f.write("\t".join(col_name_list)+"\n")
        for acc_id in output_dict:
            info_list = []
            for i in output_dict[acc_id]:
                if i is None:
                    info_list.append('None')
                else:
                    str(i)
            f.write("\t".join(info_list)+"\n")

    renew_num = 0
    for acc_id in record_dict:
        for i in record_dict[acc_id].para_dict["files"]:
            if i in download_type:
                url_path, file_path, md5_string, log_tmp = record_dict[acc_id].para_dict["files"][i]
                if log_tmp == "renew":
                    renew_num = renew_num + 1

    if not renew_num == 0:
        raise ValueError(
            "Not all NCBI data have been download! Please check! ")


@retry(tries=5)
def if_assembly_annotated(genome_record):
    ftp_path = genome_record.ftp_path
    ftp_dir_path = ftp_path.replace("ftp://ftp.ncbi.nlm.nih.gov", "")
    pt_file_url = ftp_dir_path + "/" + \
        ftp_path.split("/")[-1] + "_translated_cds.faa.gz"

    annotated_flag = True
    try:
        ftp = FTP('ftp.ncbi.nlm.nih.gov')
        ftp.login()
        ftp.dir(pt_file_url)
    except:
        error_mess = sys.exc_info()[1].args[0]

        if 'timeout' in error_mess:
            raise EnvironmentError("Check your network")
        elif 'No such file or directory' in error_mess:
            annotated_flag = False

    return annotated_flag


def ncbi_genome_chooser(args):

    log_print("Read metadata")

    if args.assembly_summary_refseq_file is None:
        args.assembly_summary_refseq_file = args.ncbi_genome_dir + \
            "/assembly_summary_refseq.txt"
        args.assembly_summary_genbank_file = args.ncbi_genome_dir + \
            "/assembly_summary_genbank.txt"

    record_dict = build_ncbi_genome_database(
        args.assembly_summary_genbank_file, args.assembly_summary_refseq_file)

    if args.taxonomy_dir:
        tax_db_file = args.taxonomy_dir+"/taxdump_xyx.db"
        if not os.path.exists(tax_db_file):
            tax_record_dict = build_taxon_database(args.taxonomy_dir)
            store_tax_record_into_sqlite(tax_record_dict, tax_db_file)
    else:
        tax_db_file = taxon_dump+"/taxdump_xyx.db"

    log_print("%d record in metadata" % len(record_dict))

    log_print("Choose by metadata")
    ncbi_rank_record = genome_chooser(record_dict, tax_db_file, taxonomic_rank=args.taxonomic_rank,
                                      keep_no_rank=(not args.remove_no_rank), top_taxon=args.top_taxon)
    log_print("%d used in from choose in metadata" % len(ncbi_rank_record))

    for tax_id in ncbi_rank_record:
        good_record = []
        for record_tmp in ncbi_rank_record[tax_id]:
            if record_tmp.del_flag == 'Good Record':
                good_record.append(record_tmp)
        ncbi_rank_record[tax_id] = good_record

    log_print("Choose by annotation")
    if not args.allow_no_annotation:
        for tax_id in ncbi_rank_record:
            for record_tmp in ncbi_rank_record[tax_id]:
                ftp_path = record_tmp.ftp_path
                ftp_dir_path = ftp_path.replace(
                    "ftp://ftp.ncbi.nlm.nih.gov", "")
                data_dir_tmp = args.ncbi_genome_dir + "/" + record_tmp.assembly_accession
                file_path = {
                    "gff_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_genomic.gff.gz",
                                 data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_genomic.gff.gz"),
                    "pt_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_translated_cds.faa.gz",
                                data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_translated_cds.faa.gz"),
                    "cds_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_cds_from_genomic.fna.gz",
                                 data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_cds_from_genomic.fna.gz"),
                    "genome_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_genomic.fna.gz",
                                    data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_genomic.fna.gz"),
                    "md5_file": (ftp_dir_path + "/" + "md5checksums.txt", data_dir_tmp + "/" + "md5checksums.txt")
                }
                para_dict_tmp = {
                    "speci_accession": re.search("(^\S+\_\d+)\.\d+", record_tmp.assembly_accession).group(1),
                    "assembly_accession": record_tmp.assembly_accession,
                    "full_accession": ftp_path.split("/")[-1],
                    "files": file_path,
                    "ftp_dir_path": ftp_dir_path,
                    "data_dir_tmp": data_dir_tmp
                }
                record_tmp.para_dict = para_dict_tmp

                md5_file_path = file_path["md5_file"][1]

                if os.path.exists(md5_file_path) and os.path.getsize(md5_file_path) != 0:
                    if len(record_tmp.para_dict["files"]['pt_file']) > 2:
                        record_tmp.annotated = True
                    else:
                        record_tmp.annotated = False
                else:
                    if if_assembly_annotated(record_tmp):
                        record_tmp.annotated = True

                        md5checksums_checker(record_tmp.para_dict,
                                             args.renew_md5_flag, download_way=args.download_way)

                        para_dict_tmp = record_tmp.para_dict
                        md5_file = para_dict_tmp["files"]["md5_file"][1]
                        md5_list = tsv_file_parse(md5_file, seq=r' ')
                        for i in para_dict_tmp["files"]:
                            for j in md5_list:
                                md5_string, nothing, file_name = md5_list[j]
                                if para_dict_tmp["files"][i][0].split("/")[-1] == file_name.split("/")[-1]:
                                    record_tmp.para_dict["files"][i] = tuple(
                                        list(record_tmp.para_dict["files"][i]) + [md5_string])
                    else:
                        record_tmp.annotated = False

    log_print("Make report")

    # make report
    col_name_list = ["assembly_accession", "species_taxid", "kingdom", "order",
                     "family", "genus", "organism_name", "refseq_category", "group", "ftp_path"]

    report_file = "chosen.genome.txt"

    tax_record_dict = read_tax_record_dict_db(tax_db_file)

    with open(report_file, 'w') as f:
        f.write("\t".join(col_name_list) + "\n")

        for tax_id in ncbi_rank_record:
            choose_record = []
            for record_tmp in ncbi_rank_record[tax_id]:
                if record_tmp.del_flag == 'Good Record':
                    if len(choose_record) < args.top_num:
                        if args.allow_no_annotation is True:
                            choose_record.append(record_tmp)
                        elif args.allow_no_annotation is False:
                            if record_tmp.annotated:
                                choose_record.append(record_tmp)
                        
            for record_tmp in choose_record:

                if record_tmp.species_taxid in tax_record_dict:

                    taxon_tmp = tax_record_dict[record_tmp.species_taxid]
                    kingdom = tax_record_dict[taxon_tmp.kingdom].sci_name if hasattr(
                        taxon_tmp, 'kingdom') else "None"
                    order = tax_record_dict[taxon_tmp.order].sci_name if hasattr(
                        taxon_tmp, 'order') else "None"
                    family = tax_record_dict[taxon_tmp.family].sci_name if hasattr(
                        taxon_tmp, 'family') else "None"
                    genus = tax_record_dict[taxon_tmp.genus].sci_name if hasattr(
                        taxon_tmp, 'genus') else "None"

                f.write("\t".join([record_tmp.assembly_accession, record_tmp.species_taxid, kingdom, order, family, genus,
                                   record_tmp.organism_name, record_tmp.refseq_category, record_tmp.group, record_tmp.ftp_path]) + "\n")


if __name__ == "__main__":

    # Download some genome from NCBI

    class abc():
        pass

    args = abc()

    args.ncbi_genome_dir = "/lustre/home/xuyuxing/Database/NCBI/genome"
    args.list_of_accession_ID = "/lustre/home/xuyuxing/Database/NCBI/genome/download.list"
    args.taxonomy_dir = None
    args.assembly_summary_refseq_file = None
    args.assembly_summary_genbank_file = None
    args.download_type = "pt_file,gff_file"
    args.renew_md5_flag = False
    args.download_way = "wget"

    ncbi_genome_downloader(args)

    # Download some genome from NCBI

    class abc():
        pass

    args = abc()

    args.ncbi_genome_dir = "/lustre/home/xuyuxing/Database/NCBI/genome"
    args.assembly_summary_refseq_file = None
    args.assembly_summary_genbank_file = None
    args.taxonomy_dir = None
    args.taxonomic_rank = 'family'
    args.top_taxon = 'Magnoliopsida'
    args.remove_no_rank = False
    args.allow_no_annotation = False
    args.renew_md5_flag = False
    args.download_way = 'wget'
    args.top_num = 2

    ncbi_genome_chooser(args)



"""
This module work to help load data to sqlite.
usage:
#
import lib.sqlite_command as sql
#
class:

function:
    check_sql_table
    check_table_columns
    init_sql_db
    init_sql_db_many_table
    insert_one_record_to_sql_table
    dict2sql_table
    sql_table2dict
    sql_table_row_num
log:
    Yuxing Xu
    2018.02.05  new module
    2018.02.05  update module
"""
import sqlite3
import codecs
import pickle
from config import sqlite_temp_store_directory

# from retry import retry


def check_sql_table(db_file):
    conn = sqlite3.connect(db_file)
    table_list = conn.execute(
        'SELECT name FROM sqlite_master WHERE type=\'table\'').fetchall()
    conn.close()

    output = []
    for i in table_list:
        output.append(i[0])
    return tuple(output)


def check_table_columns(db_file, table_name):
    conn = sqlite3.connect(db_file)
    content = conn.execute('PRAGMA table_info(' + table_name + ')').fetchall()
    conn.close()

    table_head = []
    for i in content:
        table_head.append(i[1])
    return tuple(table_head)


def all_database_stat(db_file):
    conn = sqlite3.connect(db_file)
    table_list = conn.execute(
        'SELECT name FROM sqlite_master WHERE type=\'table\'').fetchall()

    output = []
    for i in table_list:
        output.append(i[0])
    output = tuple(output)

    db_stat = {}
    for i in output:
        db_stat[i] = {}
        table_record_num = conn.execute('SELECT count(*) FROM ' + i).fetchall()
        db_stat[i]["record_num"] = table_record_num[0][0]
        content = conn.execute('PRAGMA table_info(' + i + ')').fetchall()
        table_head = []
        for j in content:
            table_head.append(j[1])
        db_stat[i]["column"] = table_head

    conn.close()
    return db_stat


def init_sql_db(db_file, table_name, column_name_list, remove_old_db=True):
    import os
    from lib.common.os import cmd_run
    if remove_old_db is True:
        if os.path.exists(db_file):
            cmd_run("rm %s" % db_file)

    conn = sqlite3.connect(db_file)

    create_cmd = '''CREATE TABLE ''' + table_name + " ("
    for column_name in column_name_list:
        create_cmd = create_cmd + "\"" + column_name + "\", "
    create_cmd = create_cmd.rstrip(", ")
    create_cmd = create_cmd + ")"
    conn.execute(create_cmd)

    conn.commit()
    conn.close()


def init_sql_db_many_table(db_file, table_columns_dict, remove_old_db=True):
    import os
    from lib.common.os import cmd_run
    if remove_old_db and os.path.exists(db_file):
        cmd_run("rm %s" % db_file)

    conn = sqlite3.connect(db_file)

    for table_name in table_columns_dict:
        create_cmd = "CREATE TABLE \"" + table_name + "\" ("
        for column_name in table_columns_dict[table_name]:
            create_cmd = create_cmd + column_name + ", "
        create_cmd = create_cmd.rstrip(", ")
        create_cmd = create_cmd + ")"
        conn.execute(create_cmd)

    conn.commit()
    conn.close()


def insert_one_record_to_sql_table(data_tuple, columns_list_tuple, db_file, table_name):
    conn = sqlite3.connect(db_file)
    insert_cmd = '''INSERT INTO ''' + table_name + " " + tuple(columns_list_tuple).__str__() \
                 + ''' VALUES ''' + tuple(data_tuple).__str__()
    conn.execute(insert_cmd)
    conn.commit()
    conn.close()


def dict2sql_table(data_dict, table_columns_dict, db_file, table_name):
    column_name = table_columns_dict[table_name]

    if len(column_name) != len(data_dict[0]):
        raise ValueError

    def iter_sql_insert(data_dict):
        for i in data_dict:
            table_insert = data_dict[i]
            yield tuple(table_insert)

    conn = sqlite3.connect(db_file)
    insert_cmd = '''INSERT INTO ''' + table_name + \
        ''' VALUES (''' + "?," * len(column_name)
    insert_cmd = insert_cmd.rstrip(",")
    insert_cmd = insert_cmd + ")"
    conn.executemany(insert_cmd, iter_sql_insert(data_dict))
    conn.commit()
    conn.close()


def sql_table2dict(db_file, table_name, top=0, key="", print_if=False):
    table_head = check_table_columns(db_file, table_name)

    conn = sqlite3.connect(db_file)
    if not top == 0:
        content = conn.execute(
            'SELECT * FROM ' + table_name + ' LIMIT ?', (top,)).fetchall()
    else:
        content = conn.execute('SELECT * FROM ' + table_name).fetchall()

    conn.close()

    output_dict = {}
    num = 0
    for i in content:
        num = num + 1
        if key == "":
            output_dict[num] = i
        else:
            key_value = i[table_head.index(key)]
            output_dict[key_value] = i

    if print_if is True:
        printer = ""
        for i in table_head:
            printer = printer + i + "\t"
        printer = printer.rstrip("\t") + "\n" + "---" + "\n"
        for i in content:
            for j in i:
                printer = printer + str(j) + "\t"
            printer = printer.rstrip("\t") + "\n"

    return output_dict, table_head


def sql_table_row_num(db_file, table_name):
    conn = sqlite3.connect(db_file)
    record_len = conn.execute("select count(*) from %s" %
                              table_name).fetchall()[0][0]
    conn.close()
    return record_len


def drop_index(db_file_name, index_name):
    conn = sqlite3.connect(db_file_name)
    conn.execute("DROP INDEX %s" % index_name)
    conn.commit()
    conn.close()


def build_index(db_file_name, table_name, key_col_name):
    conn = sqlite3.connect(db_file_name)
    conn.execute("PRAGMA temp_store_directory = \'%s\'" %
                 sqlite_temp_store_directory)
    conn.execute("CREATE UNIQUE INDEX %s_index on %s (\"%s\")" %
                 (table_name, table_name, key_col_name))
    conn.commit()
    conn.close()


def drop_table(db_file_name, table_name):
    conn = sqlite3.connect(db_file_name)
    conn.execute("DROP TABLE %s" % table_name)
    conn.commit()
    conn.close()


def dict_context_yield(list_input):
    for query in list_input:
        yield query


# @retry()
def sqlite_write(record_list, db_file, table_name, columns_list):
    """
    :param record_list:
            record_list=[("A1","B1","C1","D1","E1"),("A2","B2","C2","D2","E2"),...]
    :param db_file:
    :param table_name: table should be inited
    :param columns_list:
            columns_list = ["A","B","C","D","E"]
    :return:
    """
    conn = sqlite3.connect(db_file)
    insert_cmd = '''INSERT INTO "''' + table_name + \
        '''" VALUES (''' + "?," * len(columns_list)
    insert_cmd = insert_cmd.rstrip(",")
    insert_cmd = insert_cmd + ")"
    conn.execute("PRAGMA temp_store_directory = \'%s\'" %
                 sqlite_temp_store_directory)
    conn.executemany(insert_cmd, dict_context_yield(record_list))

    conn.commit()
    conn.close()


def dict_context_yield_for_update(list_input):
    for key, value in list_input:
        yield (value, key)

# @retry()


def sqlite_update(db_file, table_name, key_col, value_col, key_value_tuple_list):
    """
    :param key_col = 'name'
    :param value_col = 'age'
    :param key_value_tuple: [("Mark Arnold", 30)]
    :return:
    """
    conn = sqlite3.connect(db_file)
    update_cmd = '''UPDATE %s SET %s = ? WHERE %s = ?''' % (
        table_name, value_col, key_col)

    conn.executemany(
        update_cmd, dict_context_yield_for_update(key_value_tuple_list))

    conn.commit()
    conn.close()


def sqlite_delete(db_file, table_name, key_name=None, value_tuple=None):
    if key_name is None:
        conn = sqlite3.connect(db_file)
        # print("SELECT %s FROM %s" % (column_string, table_name))
        content = conn.execute("DELETE FROM \"%s\"" %
                               (table_name)).fetchall()
        conn.commit()
        conn.close()

    else:
        conn = sqlite3.connect(db_file)
        value_tuple = tuple(value_tuple)
        if len(value_tuple) == 1:
            content = conn.execute(
                "DELETE FROM \"%s\" WHERE \"%s\" = ?" % (
                    table_name, key_name),
                value_tuple).fetchall()
        else:
            print("DELETE FROM \"%s\" WHERE \"%s\" IN " % (
                table_name, key_name) + value_tuple.__str__())
            content = conn.execute(
                "DELETE FROM \"%s\" WHERE \"%s\" IN " % (
                    table_name, key_name) + value_tuple.__str__()).fetchall()
        conn.commit()
        conn.close()


def sqlite_select_by_a_key(db_file, table_name, key_name, value_tuple):
    conn = sqlite3.connect(db_file)
    if len(value_tuple) == 0:
        content = conn.execute(
            "SELECT * FROM %s" % table_name).fetchall()
    elif len(value_tuple) == 1:
        content = conn.execute(
            "SELECT * FROM %s WHERE \"%s\" = ?" % (table_name, key_name), value_tuple).fetchall()
    else:
        content = conn.execute(
            "SELECT * FROM %s WHERE \"%s\" IN " % (table_name, key_name) + value_tuple.__str__()).fetchall()
    conn.close()
    return content


def sqlite_select(db_file, table_name, column_list=None, key_name=None, value_tuple=None):
    if column_list is None:
        column_string = "*"
    else:
        column_string = ""
        for i in column_list:
            column_string = column_string + i + ","
        column_string = column_string.rstrip(",")

    if key_name is None:
        conn = sqlite3.connect(db_file)
        # print("SELECT %s FROM %s" % (column_string, table_name))
        content = conn.execute("SELECT %s FROM \"%s\"" %
                               (column_string, table_name)).fetchall()
        conn.close()
        return content
    else:
        conn = sqlite3.connect(db_file)
        value_tuple = tuple(value_tuple)
        if len(value_tuple) == 1:
            content = conn.execute(
                "SELECT %s FROM \"%s\" WHERE \"%s\" = ?" % (
                    column_string, table_name, key_name),
                value_tuple).fetchall()
        else:
            content = conn.execute(
                "SELECT %s FROM \"%s\" WHERE \"%s\" IN " % (
                    column_string, table_name, key_name) + value_tuple.__str__()).fetchall()
        conn.close()
        return content


def sqlite_execute(cmd_string, db_file, commit=False):
    conn = sqlite3.connect(db_file)
    content = conn.execute(cmd_string).fetchall()
    if commit:
        conn.commit()
    conn.close()
    return content


def build_database(data_generator, parse_function, table_columns_dict, sql3_db_file):
    import time
    from lib.xuyuxing.base.common_command import log_print
    from collections import OrderedDict

    start_time = time.time()
    init_sql_db_many_table(sql3_db_file, table_columns_dict)

    record_dict = OrderedDict()
    num = 0
    for record in data_generator:
        record_dict[num] = parse_function(record)
        num += 1

        if len(record_dict) % 10000 == 0:
            for table_name in table_columns_dict:
                record_tuple_now = [record_dict[i][table_name] for i in record_dict]
                sqlite_write(record_tuple_now, sql3_db_file, table_name, table_columns_dict[table_name])
            record_dict = OrderedDict()
        
        round_time = time.time()
        if round_time - start_time > 10:
            log_print("%d finished" % (num))
            start_time = round_time

    if len(record_dict) > 0:
        for table_name in table_columns_dict:
            record_tuple_now = [record_dict[i][table_name] for i in record_dict]
            sqlite_write(record_tuple_now, sql3_db_file, table_name, table_columns_dict[table_name])

    record_dict = OrderedDict()
    log_print("%d finished" % (num))

    return sql3_db_file

# sqlite + pickle


def pickle_dump_obj(unpickled_obj):
    pickled = codecs.encode(pickle.dumps(unpickled_obj), "base64").decode()
    return pickled


def pickle_load_obj(pickled_string):
    unpickled_obj = pickle.loads(
        codecs.decode(pickled_string.encode(), "base64"))
    return unpickled_obj


if __name__ == '__main__':
    db_file = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/all_map/test/Cau/test.db'

    init_sql_db(db_file, 'table1', ['name', 'sex', 'age'], True)

    record_list = [
        ("Mark Arnold", "M", "28"),
        ("Barbara Jackson", "W", "29"),
        ("Wilhelm Schacht", "M", "")
    ]

    sqlite_write(record_list, db_file, 'table1', ['name', 'sex', 'age'])

    sqlite_update(db_file, 'table1', 'name', 'age', [
                  ("Wilhelm Schacht", 50), ("Mark Arnold", 29)])

    sqlite_delete(db_file, 'table1', 'name',
                  ("Mark Arnold", "Barbara Jackson"))

    # for load a big database into a sqlite
    from Bio import SeqIO
    xml_file = "/lustre/home/xuyuxing/Database/Uniprot/sport/uniprot_sprot.xml"
    sql3_db_file = "/lustre/home/xuyuxing/Database/Uniprot/sport/uniprot_sprot.db"

    table_columns_dict = {'record':['id', 'codenstring']}

    data_generator = SeqIO.parse(xml_file, "uniprot-xml")

    def parse_function(seqio_obj):
        return {'record': (seqio_obj.id, pickle_dump_obj(seqio_obj))} 

    build_database(data_generator, parse_function, table_columns_dict, sql3_db_file)

    build_index(sql3_db_file, 'record', 'id')


import math
import os
import re
from lib.common.os import mkdir, cmd_run, multiprocess_running
from lib.common.genome.seq_base import read_fasta_big
from api.common.genome.blast import outfmt6_read_big
from lib.xuyuxing.base.base_function import merge_file

def w_index(evalue):
    if evalue == 0.0:
        return 100
    else:
        return min(100, round(-math.log(evalue, 10) / 2))

def hcluster_pipeline(pt_file_dir, output_dir, threads=56, option_string="-m 750 -w 0 -s 0.34 -O"):
    mkdir(output_dir, True)
    rename_seq_dir = output_dir + "/rename_seq"
    mkdir(rename_seq_dir, True)
    diamond_dir = output_dir + "/diamond"
    mkdir(diamond_dir, True)

    # rename step

    rename_dict = {}
    seq_file_dict = {}
    new_seq_species_dict = {}
    f_n = 0
    for f_name in os.listdir(pt_file_dir):
        p1 = r'(.*)\.fasta'
        p2 = r'(.*)\.fa'
        if re.match(p1, f_name) or re.match(p2, f_name):
            mbj = re.match(p1, f_name) or re.match(p2, f_name)
            prefix = mbj.groups()[0]
            input_f_name = pt_file_dir + "/" + f_name
            output_f_name = rename_seq_dir + "/" + prefix+".re.fa"
            bls_f_name = diamond_dir + "/" + prefix+".bls"
            seq_file_dict[prefix] = {'raw':input_f_name ,'rename':output_f_name ,'bls':bls_f_name}
            r_n = 0
            with open(output_f_name, 'w') as f:
                for r in read_fasta_big(input_f_name):
                    new_seq_id = str(f_n) + "_" + str(r_n)
                    rename_dict[new_seq_id] = r.seqname
                    new_seq_species_dict[new_seq_id] = prefix
                    f.write(">%s\n%s" % (new_seq_id, r.wrap()))
                    r_n += 1
            f_n += 1
            
    rename_map_file = rename_seq_dir + "/rename.map"
    with open(rename_map_file, 'w') as f:
        for i in rename_dict:
            f.write("%s\t%s\n" % (rename_dict[i], i))
    
    # diamond
    ## makedb

    # args_list = []
    # for sp_id in seq_file_dict:
    #     cmd_string = "diamond makedb --in %s --db %s" % (seq_file_dict[sp_id]['rename'], seq_file_dict[sp_id]['db'])
    #     args_list.append((cmd_string, None, 1, True, None))

    # multiprocess_running(cmd_run, args_list, threads, None, True)

    merged_fasta = diamond_dir + "/merged.fa"
    merge_file([seq_file_dict[i]['rename'] for i in seq_file_dict], merged_fasta)

    merged_db = diamond_dir + "/merged.dmnd"
    cmd_string = "diamond makedb --in %s --db %s" % (merged_fasta, merged_db)
    cmd_run(cmd_string, None, 1, True, None)

    ## run blastp
    args_list = []
    for sp_id in seq_file_dict:
        q_file = seq_file_dict[sp_id]['rename']
        bls_file = seq_file_dict[sp_id]['bls']
        cmd_string = "diamond blastp -d %s -q %s -o %s --more-sensitive -p 8 --quiet -e 1e-5" % (merged_db, q_file, bls_file)
        args_list.append((cmd_string, None, 1, True, None))
        # print(cmd_string)
    multiprocess_running(cmd_run, args_list, threads, None, True)

    # w-value
    hcluster_dir = output_dir + "/hcluster"
    mkdir(hcluster_dir, True)

    for sp_id in seq_file_dict:
        bls_file = seq_file_dict[sp_id]['bls']
        wvalue_file = hcluster_dir + "/" + sp_id+".wvalue"
        seq_file_dict[sp_id]['wvalue'] = wvalue_file

        with open(wvalue_file, 'w') as f:
            for r in outfmt6_read_big(bls_file):
                for hit in r.hit:
                    hsp = hit.hsp[0]
                    q_id = hsp.query.qID
                    s_id = hsp.subject.Hit_id
                    evalue = hsp.Hsp_evalue
                    wvalue = w_index(evalue)
                    f.write("%s\t%s\t%d\n" % (q_id, s_id, wvalue))

    # hcluster running
    merged_wvalue = hcluster_dir + "/merged.wvalue"
    merge_file([seq_file_dict[i]['wvalue'] for i in seq_file_dict], merged_wvalue)

    hcluster_outfile = hcluster_dir + "/hcluster.out"
    cmd_string = 'hcluster_sg %s -o %s %s' % (option_string, hcluster_outfile, merged_wvalue)
    cmd_run(cmd_string, None, 1, True, None)

    # convert hcluster out to tsv
    all_id_list = []
    with open(hcluster_outfile, 'r') as fi:
        for each_line in fi:
            each_line = each_line.strip()
            info = each_line.split('\t')
            id_list = info[6].split(",")
            id_list.remove("")
            all_id_list.append(id_list)

    all_id_list = sorted(all_id_list, key=lambda x:len(x), reverse=True)

    num = 0

    sp_id_list = list(seq_file_dict.keys())

    cluster_dict = {}
    for cluster_tmp in all_id_list:
        id_tmp = "hcluster%d" % num
        cluster_dict[id_tmp] = {i:[] for i in sp_id_list}

        for sp_id in sp_id_list:
            cluster_dict[id_tmp][sp_id] = [rename_dict[i] for i in cluster_tmp if new_seq_species_dict[i] == sp_id]

        num += 1
    
    # add uncluster seqid

    clustered_gene_dict = {i:[] for i in sp_id_list}
    for sp_id in sp_id_list:
        clustered_gene_dict[sp_id] = []
        for id_tmp in cluster_dict:
            clustered_gene_dict[sp_id] += cluster_dict[id_tmp][sp_id]
    
    unclustered_gene_dict = {}
    for sp_id in sp_id_list:
        all_gene_list = [rename_dict[i] for i in new_seq_species_dict if new_seq_species_dict[i] == sp_id]
        unclustered_gene_dict[sp_id] = list(set(all_gene_list) - set(clustered_gene_dict[sp_id]))
        
    for sp_id in sp_id_list:
        for gene_id in unclustered_gene_dict[sp_id]:
            id_tmp = "hcluster%d" % num
            cluster_dict[id_tmp] = {i:[] for i in sp_id_list}
            cluster_dict[id_tmp][sp_id] = [gene_id]

            num += 1

    # write tsv
    tsv_file = output_dir + '/hcluster.tsv'
    with open(tsv_file, 'w') as f:
        f.write("\t".join(['Orthogroup'] + sp_id_list) + "\n")
        for id_tmp in cluster_dict:
            printer_list = []
            for sp_id in sp_id_list:
                printer_list.append(", ".join(cluster_dict[id_tmp][sp_id]))
            printer_string = "\t".join([id_tmp] + printer_list)
            f.write(printer_string+"\n")
            
    # write GeneCount tsv
    tsv_file = output_dir + '/hcluster.GeneCount.tsv'
    with open(tsv_file, 'w') as f:
        f.write("\t".join(['Orthogroup'] + sp_id_list) + "\n")
        for id_tmp in cluster_dict:
            printer_list = []
            for sp_id in sp_id_list:
                printer_list.append(str(len(cluster_dict[id_tmp][sp_id])))
            printer_string = "\t".join([id_tmp] + printer_list)
            f.write(printer_string+"\n")
            







import sys
import re
from lib.common.math.interval import section, group_by_intervals_with_overlap_threshold
from lib.common.genome.seq_base import BioSeq, read_fasta_by_faidx
from lib.xuyuxing.base.common_command import merge_dict
from interlap import InterLap
from BCBio import GFF
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation


def fancy_name_parse(input_string):
    match_obj = re.match(r'^(\S+):(\d+)-(\d+)$', input_string)
    if match_obj:
        contig_name, c_start, c_end = match_obj.groups()
        strand = "."

    match_obj = re.match(r'^(\S+):(\d+)-(\d+):(\S+)$', input_string)
    if match_obj:
        contig_name, c_start, c_end, strand = match_obj.groups()

    start = min(int(c_start), int(c_end))
    end = max(int(c_start), int(c_end))

    return contig_name, start, end, strand


class ChrLoci(object):
    def __init__(self, chr_id=None, strand=None, start=None, end=None, chrome=None):
        self.chr_id = chr_id
        self.chrome = chrome

        if strand is None:
            self.strand = strand
        elif strand == "+" or str(strand) == '1':
            self.strand = "+"
        elif strand == "-" or str(strand) == '-1':
            self.strand = "-"

        if end is not None and start is not None:
            self.start = min(int(start), int(end))
            self.end = max(int(start), int(end))
            self._range = (self.start, self.end)
            self.length = abs(self.end - self.start) + 1

    # def __eq__(self, other):
    #     """Implement equality by comparing all the location attributes."""
    #     if not isinstance(other, ChrLoci):
    #         return False
    #     return self.start == other.start and \
    #            self.end == other.end and \
    #            self.strand == other.strand and \
    #            self.chr_id == other.chr_id

    def get_fancy_name(self):
        if self.strand is None:
            self.fancy_name = "%s:%d-%d" % (self.chr_id, self.start, self.end)
        else:
            self.fancy_name = "%s:%d-%d:%s" % (self.chr_id, self.start, self.end, self.strand)
        return self.fancy_name

    def __str__(self):
        try:
            self.get_fancy_name()
            return self.fancy_name
        except:
            return "No detail range"

    @property
    def range(self):
        return self._range

    @range.setter
    def range(self, value):
        start, end = value
        if end is not None and start is not None:
            self.start = min(int(start), int(end))
            self.end = max(int(start), int(end))
            self._range = (self.start, self.end)

    def get_sequence(self, data_fasta_file, RNA=False):
        sequence = read_fasta_by_faidx(data_fasta_file)[self.chr_id].sub(self._range[0], self._range[1], self.strand,
                                                                         RNA)
        self.get_fancy_name()
        return BioSeq(sequence, self.fancy_name)


def section_of_chr_loci(chr_loci1, chr_loci2):
    if chr_loci1.chr_id == chr_loci2.chr_id:
        if (not chr_loci1.strand is None) and (not chr_loci2.strand is None):
            if chr_loci1.strand == chr_loci2.strand:
                return section(chr_loci1.range, chr_loci2.range, int_flag=True)
            else:
                return False, None
        else:
            return section(chr_loci1.range, chr_loci2.range, int_flag=True)
    else:
        return False, None


def cluster_of_chr_loci(chr_loci_list, overlap_threshold=0.0, use_strand=True):
    if use_strand:
        data_dict = {}
        for chr_loci in chr_loci_list:
            if chr_loci.chr_id not in data_dict:
                data_dict[chr_loci.chr_id] = {"+": {}, "-": {}, ".": {}}
            data_dict[chr_loci.chr_id][chr_loci.strand][chr_loci] = (chr_loci.start, chr_loci.end)
    else:
        data_dict = {}
        for chr_loci in chr_loci_list:
            if chr_loci.chr_id not in data_dict:
                data_dict[chr_loci] = {".": {}}
            data_dict[chr_loci.chr_id][chr_loci.strand][chr_loci] = (chr_loci.start, chr_loci.end)

    for chr_id in data_dict:
        for strand in data_dict[chr_id]:
            data_dict[chr_id][strand] = group_by_intervals_with_overlap_threshold(data_dict[chr_id][strand],
                                                                                  overlap_threshold=overlap_threshold)

    return data_dict


class Genome(object):
    def __init__(self, id=None, species=None, version=None, genome_file=None, gff_file=None, cds_file=None,
                 cDNA_file=None, aa_file=None):
        self.id = id
        self.species = species
        self.version = version
        self.genome_file = genome_file
        self.gff_file = gff_file
        self.cds_file = cds_file
        self.cDNA_file = cDNA_file
        self.aa_file = aa_file

    def genome_file_parse(self):
        self.genome_seq = read_fasta_by_faidx(self.genome_file)

    def genome_feature_parse(self, load_seq_info=False):
        # check needed files
        if not hasattr(self, 'gff_file') or self.gff_file is None:
            raise ValueError("genome_feature_parse need gff_file!")

        if load_seq_info:
            if not hasattr(self, 'genome_file') or self.genome_file is None:
                if not hasattr(self, 'cDNA_file') or self.cds_file is None:
                    raise ValueError("load_seq_info need genome_file or cDNA_file!")

        self.chromosome = {}
        feature_dict_list = []
        with open(self.gff_file, 'r') as in_handle:
            for rec in GFF.parse(in_handle):
                self.chromosome[rec.id] = Chromosome(id=rec.id, species=self.species, version=self.version,
                                                     sub_feature=rec.features)
                feature_dict = self.chromosome[rec.id].make_feature_index()
                feature_dict_list.append(feature_dict)
        self.feature_dict = merge_dict(feature_dict_list, True)

        tmp_dict = {}
        for i in self.feature_dict:
            tmp_dict[i] = {}
            for j in self.feature_dict[i]:
                tmp_dict[i][j.name] = j

        self.feature_dict = tmp_dict

    def search(self, fancy_name_string):
        contig_name, c_start, c_end, strand = fancy_name_parse(fancy_name_string)
        if strand == ".":
            feature_index_list = ["+", "-", "."]
        else:
            feature_index_list = [strand]

        output_dir = {}
        for strand_used in feature_index_list:
            feature_index = self.chromosome[contig_name].feature_index[strand_used]
            for f_s, f_e, feature in feature_index.find((c_start, c_end)):
                output_dir[feature.name] = feature

        return output_dir


class Chromosome(object):
    def __init__(self, id=None, species=None, version=None, seq=None, sub_feature=None):
        self.id = id
        self.species = species
        self.version = version
        self.seq = seq
        self.sub_feature = sub_feature

    def make_feature_index(self):
        if not hasattr(self, 'sub_feature'):
            raise ValueError("make_feature_index need sub_feature!")

        self.feature_index = {
            "+": InterLap(),
            "-": InterLap(),
            ".": InterLap()
        }

        feature_dict = {}

        for feature in self.sub_feature:
            start_tmp = feature.location.start.position + 1
            end_tmp = feature.location.end.position
            start = min(start_tmp, end_tmp)
            end = max(start_tmp, end_tmp)
            chr_loci = ChrLoci(chr_id=self.id, strand=feature.strand, start=start, end=end, chrome=self)

            if feature.strand == 1:
                index_tmp = self.feature_index['+']
            elif feature.strand == -1:
                index_tmp = self.feature_index['-']
            else:
                index_tmp = self.feature_index['.']

            gene_xyx = GenomeFeature(feature.id, type=feature.type, chr_loci=chr_loci, sub_feature=feature.sub_features)

            index_tmp.add((start, end, gene_xyx))

            if feature.type not in feature_dict:
                feature_dict[feature.type] = []

            feature_dict[feature.type].append(gene_xyx)

        return feature_dict


# class GenomeFeature(object):
#     def __init__(self, name, type=None, chr_loci=None, sub_feature=None):
#         self.name = name
#         self.type = type
#         self.chr_loci = chr_loci
#         self.sub_feature = sub_feature

class GenomeFeature(ChrLoci):
    def __init__(self, name=None, type=None, chr_loci=None, sub_feature=None, chr_id=None, strand=None, start=None,
                 end=None, chrome=None, bio_location=None):
        if not chr_loci is None:
            chr_loci = chr_loci
        elif not bio_location is None:
            chr_loci = ChrLoci(chr_id=chr_id, strand=bio_location.strand, start=bio_location.start + 1,
                               end=bio_location.end)
        else:
            chr_loci = ChrLoci(chr_id=chr_id, strand=strand, start=start, end=end, chrome=chrome)
        super(GenomeFeature, self).__init__(chr_id=chr_loci.chr_id, strand=chr_loci.strand, start=chr_loci.start,
                                            end=chr_loci.end, chrome=chr_loci.chrome)
        self.name = name
        self.type = type
        self.chr_loci = chr_loci
        self.sub_feature = sub_feature


class mRNA(GenomeFeature):
    def __init__(self, name, gene_name=None, chr_loci=None, sub_feature=None, cds_seq=None, aa_seq=None, cDNA_seq=None):
        super(mRNA, self).__init__(name=name, type='mRNA', chr_loci=chr_loci, sub_feature=sub_feature)
        self.cds_seq = cds_seq
        self.aa_seq = aa_seq
        self.cDNA_seq = cDNA_seq
        self.gene_name = gene_name


class Gene(GenomeFeature):
    def __init__(self, name, species=None, chr_loci=None, sub_feature=None, model_cds_seq=None, model_aa_seq=None,
                 model_cDNA_seq=None):
        super(Gene, self).__init__(name=name, type='gene', chr_loci=chr_loci, sub_feature=sub_feature)
        self.model_cds_seq = model_cds_seq
        self.model_aa_seq = model_aa_seq
        self.model_cDNA_seq = model_cDNA_seq
        self.species = species

    def __str__(self):
        return "ID: %s" % self.name


class GeneSet(object):
    def __init__(self, name, gene_list):
        self.name = name
        self.gene_list = gene_list
        self.tree_file = None
        self.seq_file = None
        self.aln_file = None

    def speci_stat(self):
        output_dir = {}
        for i in self.gene_list:
            if i.species not in output_dir:
                output_dir[i.species] = 0
            output_dir[i.species] = output_dir[i.species] + 1
        return output_dir

    def __str__(self):
        return "ID: %s; Gene Num: %d" % (self.name, len(self.gene_list))


class SimpleRangePair(object):
    def __init__(self, rangeA, rangeB, score=None):
        """range is ChrLoci, most time rangeA is strand + """
        self.rangeA = rangeA
        self.rangeB = rangeB
        self.score = score


class BlastHspRecord(SimpleRangePair):
    def __init__(self, rangeA, rangeB, hsp_id, pid):
        super(BlastHspRecord, self).__init__(rangeA, rangeB, float(pid))
        self.query = self.rangeA
        self.subject = self.rangeB
        self.hsp_id = int(hsp_id)
        self.pid = float(pid)


class HomoPredictResults(object):
    def __init__(self, query_gene=None, subject_species=None, hit_gene_list=None):
        self.query_gene = query_gene
        self.subject_species = subject_species
        self.hit_gene_list = hit_gene_list


# some useful tools

def biogff_convert_to_GenomeFeature(biogff_object, chr_id=None):
    gf = GenomeFeature(name=biogff_object.id, type=biogff_object.type, chr_id=chr_id,
                       bio_location=biogff_object.location)
    if hasattr(biogff_object, 'sub_features') and len(biogff_object.sub_features) != 0:
        gf.sub_feature = []
        for sub_biogff_object in biogff_object.sub_features:
            gf.sub_feature.append(biogff_convert_to_GenomeFeature(sub_biogff_object, chr_id))

    return gf


def read_gff_to_feature_dict(gff_file):
    feature_dict = {}

    with open(gff_file, 'r') as in_handle:
        for rec in GFF.parse(in_handle):
            for feature in rec.features:
                new_feature = biogff_convert_to_GenomeFeature(feature, rec.id)
                feature_dict[new_feature.name] = new_feature

    return feature_dict


def extract_sub_feature_to_gff_object(GenomeFeature_Object, source):
    sub_features = []
    # print(GenomeFeature_Object.name)
    for sgf in GenomeFeature_Object.sub_feature:
        if sgf.strand == "+" or sgf.strand == 1:
            strand = 1
        elif sgf.strand == "-" or sgf.strand == -1:
            strand = -1
        else:
            strand = 0

        sub_qualifiers = {
            "source": source,
            "ID": sgf.name
        }

        if sgf.type == 'CDS':
            sub_qualifiers['phase'] = sgf.phase

        sub_feature = SeqFeature(FeatureLocation(sgf.start - 1, sgf.end, strand=strand),
                                 type=sgf.type, qualifiers=sub_qualifiers)

        if hasattr(sgf, 'sub_feature') and (not sgf.sub_feature is None):
            # print(sgf.sub_feature)
            sub_sub_features = extract_sub_feature_to_gff_object(sgf, source)
            sub_feature.sub_features = sub_sub_features

        sub_features.append(sub_feature)

    return sub_features


def write_GenomeFeature_to_GFF(GenomeFeature_list, output_gff_file, source="."):
    chr_list = list(set([gf.chr_id for gf in GenomeFeature_list]))

    rec_list = []
    for chr_id in chr_list:
        rec = SeqRecord(Seq(""), chr_id, description='')
        rec.features = []

        for gf in GenomeFeature_list:
            if gf.strand == "+" or gf.strand == 1:
                strand = 1
            elif gf.strand == "-" or gf.strand == -1:
                strand = -1
            else:
                strand = 0

            if gf.chr_id == chr_id:
                qualifiers = {
                    "source": source,
                    "ID": gf.name
                }
                top_feature = SeqFeature(FeatureLocation(gf.start - 1, gf.end, strand=strand), type=gf.type,
                                         qualifiers=qualifiers)

                top_feature.sub_features = extract_sub_feature_to_gff_object(gf, source)

                rec.features.append(top_feature)

        rec_list.append(rec)

    with open(output_gff_file, "w") as f:
        GFF.write(rec_list, f)


# todo
def get_uniq_annotation(hit_gene_list, overlap_threshold=0.5):
    loci_sort_dict = {}
    for hit_gene in hit_gene_list:
        hit_loci = hit_gene.chr_loci
        if hit_loci.chr_id not in loci_sort_dict:
            loci_sort_dict[hit_loci.chr_id] = {"+": [], "-": []}
        loci_sort_dict[hit_loci.chr_id][hit_loci.strand].append(hit_gene)

    for chr_id in loci_sort_dict:
        for strand in loci_sort_dict[chr_id]:
            group_dict = group_by_intervals_with_overlap_threshold(
                {hit_gene: hit_gene.range for hit_gene in loci_sort_dict[chr_id][strand]}, overlap_threshold)


if __name__ == '__main__':
    # build a genome database
    genome_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/Cuscuta.genome.v1.1.fasta'
    gff_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/Cuscuta.v1.1.gff3'
    cds_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/Cuscuta.cds.v1.1.fasta'
    pt_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/Cuscuta.pt.v1.1.fasta'
    cDNA_file = '/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/Cuscuta.cDNA.v1.1.fasta'

    Cau_genome = Genome(id='Cau', species='267555', version='v1.1', genome_file=genome_file, gff_file=gff_file,
                        cds_file=cds_file, cDNA_file=cDNA_file, aa_file=pt_file)

    Cau_genome.genome_feature_parse()

    all_gene_list = Cau_genome.feature_dict['gene']

    gene_list_in_range = Cau_genome.search("C000N:1-100000")

    # compare two annotation

    # just read a gff file

    gf_dict = read_gff_to_feature_dict(gff_file)



# coding: utf-8
from itertools import chain
from collections import Iterable
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from matplotlib import colors
import math

# Fork from https://github.com/tctianchi/pyvenn/blob/master/venn.py

default_colors = [
    # r, g, b, a
    [92, 192, 98, 0.5],
    [90, 155, 212, 0.5],
    [246, 236, 86, 0.6],
    [241, 90, 96, 0.4],
    [255, 117, 0, 0.3],
    [82, 82, 190, 0.2],
]
default_colors = [
    [i[0] / 255.0, i[1] / 255.0, i[2] / 255.0, i[3]]
    for i in default_colors
]


def draw_ellipse(fig, ax, x, y, w, h, a, fillcolor):
    e = patches.Ellipse(
        xy=(x, y),
        width=w,
        height=h,
        angle=a,
        color=fillcolor)
    ax.add_patch(e)


def draw_triangle(fig, ax, x1, y1, x2, y2, x3, y3, fillcolor):
    xy = [
        (x1, y1),
        (x2, y2),
        (x3, y3),
    ]
    polygon = patches.Polygon(
        xy=xy,
        closed=True,
        color=fillcolor)
    ax.add_patch(polygon)


def draw_text(fig, ax, x, y, text, color=[0, 0, 0, 1], fontsize=14, ha="center", va="center"):
    ax.text(
        x, y, text,
        horizontalalignment=ha,
        verticalalignment=va,
        fontsize=fontsize,
        color="black")


def draw_annotate(fig, ax, x, y, textx, texty, text, color=[0, 0, 0, 1], arrowcolor=[0, 0, 0, 0.3]):
    plt.annotate(
        text,
        xy=(x, y),
        xytext=(textx, texty),
        arrowprops=dict(color=arrowcolor, shrink=0, width=0.5, headwidth=8),
        fontsize=14,
        color=color,
        xycoords="data",
        textcoords="data",
        horizontalalignment='center',
        verticalalignment='center'
    )


def get_labels(data, fill=["number"], return_set_collections=False):
    """
    get a dict of labels for groups in data
    @type data: list[Iterable]
    @rtype: dict[str, str]
    input
      data: data to get label for
      fill: ["number"|"logic"|"percent"]
    return
      labels: a dict of labels for different sets
    example:
    In [12]: get_labels([range(10), range(5,15), range(3,8)], fill=["number"])
    Out[12]:
    {'001': '0',
     '010': '5',
     '011': '0',
     '100': '3',
     '101': '2',
     '110': '2',
     '111': '3'}
    """

    N = len(data)

    sets_data = [set(data[i]) for i in range(N)]  # sets for separate groups
    s_all = set(chain(*data))  # union of all sets

    # bin(3) --> '0b11', so bin(3).split('0b')[-1] will remove "0b"
    set_collections = {}
    for n in range(1, 2 ** N):
        key = bin(n).split('0b')[-1].zfill(N)
        value = s_all
        sets_for_intersection = [sets_data[i] for i in range(N) if key[i] == '1']
        sets_for_difference = [sets_data[i] for i in range(N) if key[i] == '0']
        for s in sets_for_intersection:
            value = value & s
        for s in sets_for_difference:
            value = value - s
        set_collections[key] = value

    labels = {k: "" for k in set_collections}
    if "logic" in fill:
        for k in set_collections:
            labels[k] = k + ": "
    if "number" in fill:
        for k in set_collections:
            labels[k] += str(len(set_collections[k]))
    if "percent" in fill:
        data_size = len(s_all)
        for k in set_collections:
            labels[k] += "(%.1f%%)" % (100.0 * len(set_collections[k]) / data_size)

    if return_set_collections:
        return labels, set_collections
    else:
        return labels


def venn2(labels, names=['A', 'B'], **options):
    """
    plots a 2-set Venn diagram
    @type labels: dict[str, str]
    @type names: list[str]
    @rtype: (Figure, AxesSubplot)
    input
      labels: a label dict where keys are identified via binary codes ('01', '10', '11'),
              hence a valid set could look like: {'01': 'text 1', '10': 'text 2', '11': 'text 3'}.
              unmentioned codes are considered as ''.
      names:  group names
      more:   colors, figsize, dpi, fontsize
    return
      pyplot Figure and AxesSubplot object
    """
    colors = options.get('colors', [default_colors[i] for i in range(2)])
    figsize = options.get('figsize', (9, 7))
    dpi = options.get('dpi', 96)
    fontsize = options.get('fontsize', 14)

    fig = plt.figure(0, figsize=figsize, dpi=dpi)
    ax = fig.add_subplot(111, aspect='equal')
    ax.set_axis_off()
    ax.set_ylim(bottom=0.0, top=0.7)
    ax.set_xlim(left=0.0, right=1.0)

    # body
    draw_ellipse(fig, ax, 0.375, 0.3, 0.5, 0.5, 0.0, colors[0])
    draw_ellipse(fig, ax, 0.625, 0.3, 0.5, 0.5, 0.0, colors[1])
    draw_text(fig, ax, 0.74, 0.30, labels.get('01', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.26, 0.30, labels.get('10', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.30, labels.get('11', ''), fontsize=fontsize)

    # legend
    draw_text(fig, ax, 0.20, 0.56, names[0], colors[0], fontsize=fontsize, ha="right", va="bottom")
    draw_text(fig, ax, 0.80, 0.56, names[1], colors[1], fontsize=fontsize, ha="left", va="bottom")
    leg = ax.legend(names, loc='center left', bbox_to_anchor=(1.0, 0.5), fancybox=True)
    leg.get_frame().set_alpha(0.5)

    return fig, ax


def venn3(labels, names=['A', 'B', 'C'], **options):
    """
    plots a 3-set Venn diagram
    @type labels: dict[str, str]
    @type names: list[str]
    @rtype: (Figure, AxesSubplot)
    input
      labels: a label dict where keys are identified via binary codes ('001', '010', '100', ...),
              hence a valid set could look like: {'001': 'text 1', '010': 'text 2', '100': 'text 3', ...}.
              unmentioned codes are considered as ''.
      names:  group names
      more:   colors, figsize, dpi, fontsize
    return
      pyplot Figure and AxesSubplot object
    """
    colors = options.get('colors', [default_colors[i] for i in range(3)])
    figsize = options.get('figsize', (9, 9))
    dpi = options.get('dpi', 96)
    fontsize = options.get('fontsize', 14)

    fig = plt.figure(0, figsize=figsize, dpi=dpi)
    ax = fig.add_subplot(111, aspect='equal')
    ax.set_axis_off()
    ax.set_ylim(bottom=0.0, top=1.0)
    ax.set_xlim(left=0.0, right=1.0)

    # body
    draw_ellipse(fig, ax, 0.333, 0.633, 0.5, 0.5, 0.0, colors[0])
    draw_ellipse(fig, ax, 0.666, 0.633, 0.5, 0.5, 0.0, colors[1])
    draw_ellipse(fig, ax, 0.500, 0.310, 0.5, 0.5, 0.0, colors[2])
    draw_text(fig, ax, 0.50, 0.27, labels.get('001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.73, 0.65, labels.get('010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.61, 0.46, labels.get('011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.27, 0.65, labels.get('100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.39, 0.46, labels.get('101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.65, labels.get('110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.51, labels.get('111', ''), fontsize=fontsize)

    # legend
    draw_text(fig, ax, 0.15, 0.87, names[0], colors[0], fontsize=fontsize, ha="right", va="bottom")
    draw_text(fig, ax, 0.85, 0.87, names[1], colors[1], fontsize=fontsize, ha="left", va="bottom")
    draw_text(fig, ax, 0.50, 0.02, names[2], colors[2], fontsize=fontsize, va="top")
    leg = ax.legend(names, loc='center left', bbox_to_anchor=(1.0, 0.5), fancybox=True)
    leg.get_frame().set_alpha(0.5)

    return fig, ax


def venn4(labels, names=['A', 'B', 'C', 'D'], **options):
    """
    plots a 4-set Venn diagram
    @type labels: dict[str, str]
    @type names: list[str]
    @rtype: (Figure, AxesSubplot)
    input
      labels: a label dict where keys are identified via binary codes ('0001', '0010', '0100', ...),
              hence a valid set could look like: {'0001': 'text 1', '0010': 'text 2', '0100': 'text 3', ...}.
              unmentioned codes are considered as ''.
      names:  group names
      more:   colors, figsize, dpi, fontsize
    return
      pyplot Figure and AxesSubplot object
    """
    colors = options.get('colors', [default_colors[i] for i in range(4)])
    figsize = options.get('figsize', (12, 12))
    dpi = options.get('dpi', 96)
    fontsize = options.get('fontsize', 14)

    fig = plt.figure(0, figsize=figsize, dpi=dpi)
    ax = fig.add_subplot(111, aspect='equal')
    ax.set_axis_off()
    ax.set_ylim(bottom=0.0, top=1.0)
    ax.set_xlim(left=0.0, right=1.0)

    # body
    draw_ellipse(fig, ax, 0.350, 0.400, 0.72, 0.45, 140.0, colors[0])
    draw_ellipse(fig, ax, 0.450, 0.500, 0.72, 0.45, 140.0, colors[1])
    draw_ellipse(fig, ax, 0.544, 0.500, 0.72, 0.45, 40.0, colors[2])
    draw_ellipse(fig, ax, 0.644, 0.400, 0.72, 0.45, 40.0, colors[3])
    draw_text(fig, ax, 0.85, 0.42, labels.get('0001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.68, 0.72, labels.get('0010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.77, 0.59, labels.get('0011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.32, 0.72, labels.get('0100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.71, 0.30, labels.get('0101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.66, labels.get('0110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.65, 0.50, labels.get('0111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.14, 0.42, labels.get('1000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.17, labels.get('1001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.29, 0.30, labels.get('1010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.39, 0.24, labels.get('1011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.23, 0.59, labels.get('1100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.61, 0.24, labels.get('1101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.35, 0.50, labels.get('1110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.38, labels.get('1111', ''), fontsize=fontsize)

    # legend
    draw_text(fig, ax, 0.13, 0.18, names[0], colors[0], fontsize=fontsize, ha="right")
    draw_text(fig, ax, 0.18, 0.83, names[1], colors[1], fontsize=fontsize, ha="right", va="bottom")
    draw_text(fig, ax, 0.82, 0.83, names[2], colors[2], fontsize=fontsize, ha="left", va="bottom")
    draw_text(fig, ax, 0.87, 0.18, names[3], colors[3], fontsize=fontsize, ha="left", va="top")
    leg = ax.legend(names, loc='center left', bbox_to_anchor=(1.0, 0.5), fancybox=True)
    leg.get_frame().set_alpha(0.5)

    return fig, ax


def venn5(labels, names=['A', 'B', 'C', 'D', 'E'], **options):
    """
    plots a 5-set Venn diagram
    @type labels: dict[str, str]
    @type names: list[str]
    @rtype: (Figure, AxesSubplot)
    input
      labels: a label dict where keys are identified via binary codes ('00001', '00010', '00100', ...),
              hence a valid set could look like: {'00001': 'text 1', '00010': 'text 2', '00100': 'text 3', ...}.
              unmentioned codes are considered as ''.
      names:  group names
      more:   colors, figsize, dpi, fontsize
    return
      pyplot Figure and AxesSubplot object
    """
    colors = options.get('colors', [default_colors[i] for i in range(5)])
    figsize = options.get('figsize', (13, 13))
    dpi = options.get('dpi', 96)
    fontsize = options.get('fontsize', 14)

    fig = plt.figure(0, figsize=figsize, dpi=dpi)
    ax = fig.add_subplot(111, aspect='equal')
    ax.set_axis_off()
    ax.set_ylim(bottom=0.0, top=1.0)
    ax.set_xlim(left=0.0, right=1.0)

    # body
    draw_ellipse(fig, ax, 0.428, 0.449, 0.87, 0.50, 155.0, colors[0])
    draw_ellipse(fig, ax, 0.469, 0.543, 0.87, 0.50, 82.0, colors[1])
    draw_ellipse(fig, ax, 0.558, 0.523, 0.87, 0.50, 10.0, colors[2])
    draw_ellipse(fig, ax, 0.578, 0.432, 0.87, 0.50, 118.0, colors[3])
    draw_ellipse(fig, ax, 0.489, 0.383, 0.87, 0.50, 46.0, colors[4])
    draw_text(fig, ax, 0.27, 0.11, labels.get('00001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.72, 0.11, labels.get('00010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.55, 0.13, labels.get('00011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.91, 0.58, labels.get('00100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.78, 0.64, labels.get('00101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.84, 0.41, labels.get('00110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.76, 0.55, labels.get('00111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.51, 0.90, labels.get('01000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.39, 0.15, labels.get('01001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.42, 0.78, labels.get('01010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.50, 0.15, labels.get('01011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.67, 0.76, labels.get('01100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.70, 0.71, labels.get('01101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.51, 0.74, labels.get('01110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.64, 0.67, labels.get('01111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.10, 0.61, labels.get('10000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.20, 0.31, labels.get('10001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.76, 0.25, labels.get('10010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.65, 0.23, labels.get('10011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.18, 0.50, labels.get('10100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.21, 0.37, labels.get('10101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.81, 0.37, labels.get('10110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.74, 0.40, labels.get('10111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.27, 0.70, labels.get('11000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.34, 0.25, labels.get('11001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.33, 0.72, labels.get('11010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.51, 0.22, labels.get('11011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.25, 0.58, labels.get('11100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.28, 0.39, labels.get('11101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.36, 0.66, labels.get('11110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.51, 0.47, labels.get('11111', ''), fontsize=fontsize)

    # legend
    draw_text(fig, ax, 0.02, 0.72, names[0], colors[0], fontsize=fontsize, ha="right")
    draw_text(fig, ax, 0.72, 0.94, names[1], colors[1], fontsize=fontsize, va="bottom")
    draw_text(fig, ax, 0.97, 0.74, names[2], colors[2], fontsize=fontsize, ha="left")
    draw_text(fig, ax, 0.88, 0.05, names[3], colors[3], fontsize=fontsize, ha="left")
    draw_text(fig, ax, 0.12, 0.05, names[4], colors[4], fontsize=fontsize, ha="right")
    leg = ax.legend(names, loc='center left', bbox_to_anchor=(1.0, 0.5), fancybox=True)
    leg.get_frame().set_alpha(0.5)

    return fig, ax


def venn6(labels, names=['A', 'B', 'C', 'D', 'E'], **options):
    """
    plots a 6-set Venn diagram
    @type labels: dict[str, str]
    @type names: list[str]
    @rtype: (Figure, AxesSubplot)
    input
      labels: a label dict where keys are identified via binary codes ('000001', '000010', '000100', ...),
              hence a valid set could look like: {'000001': 'text 1', '000010': 'text 2', '000100': 'text 3', ...}.
              unmentioned codes are considered as ''.
      names:  group names
      more:   colors, figsize, dpi, fontsize
    return
      pyplot Figure and AxesSubplot object
    """
    colors = options.get('colors', [default_colors[i] for i in range(6)])
    figsize = options.get('figsize', (20, 20))
    dpi = options.get('dpi', 96)
    fontsize = options.get('fontsize', 14)

    fig = plt.figure(0, figsize=figsize, dpi=dpi)
    ax = fig.add_subplot(111, aspect='equal')
    ax.set_axis_off()
    ax.set_ylim(bottom=0.230, top=0.845)
    ax.set_xlim(left=0.173, right=0.788)

    # body
    draw_triangle(fig, ax, 0.637, 0.921, 0.649, 0.274, 0.188, 0.667, colors[0])
    draw_triangle(fig, ax, 0.981, 0.769, 0.335, 0.191, 0.393, 0.671, colors[1])
    draw_triangle(fig, ax, 0.941, 0.397, 0.292, 0.475, 0.456, 0.747, colors[2])
    draw_triangle(fig, ax, 0.662, 0.119, 0.316, 0.548, 0.662, 0.700, colors[3])
    draw_triangle(fig, ax, 0.309, 0.081, 0.374, 0.718, 0.681, 0.488, colors[4])
    draw_triangle(fig, ax, 0.016, 0.626, 0.726, 0.687, 0.522, 0.327, colors[5])
    draw_text(fig, ax, 0.212, 0.562, labels.get('000001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.430, 0.249, labels.get('000010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.356, 0.444, labels.get('000011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.609, 0.255, labels.get('000100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.323, 0.546, labels.get('000101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.513, 0.316, labels.get('000110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.523, 0.348, labels.get('000111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.747, 0.458, labels.get('001000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.325, 0.492, labels.get('001001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.670, 0.481, labels.get('001010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.359, 0.478, labels.get('001011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.653, 0.444, labels.get('001100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.344, 0.526, labels.get('001101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.653, 0.466, labels.get('001110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.363, 0.503, labels.get('001111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.750, 0.616, labels.get('010000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.682, 0.654, labels.get('010001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.402, 0.310, labels.get('010010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.392, 0.421, labels.get('010011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.653, 0.691, labels.get('010100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.651, 0.644, labels.get('010101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.490, 0.340, labels.get('010110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.468, 0.399, labels.get('010111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.692, 0.545, labels.get('011000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.666, 0.592, labels.get('011001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.665, 0.496, labels.get('011010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.374, 0.470, labels.get('011011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.653, 0.537, labels.get('011100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.652, 0.579, labels.get('011101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.653, 0.488, labels.get('011110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.389, 0.486, labels.get('011111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.553, 0.806, labels.get('100000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.313, 0.604, labels.get('100001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.388, 0.694, labels.get('100010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.375, 0.633, labels.get('100011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.605, 0.359, labels.get('100100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.334, 0.555, labels.get('100101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.582, 0.397, labels.get('100110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.542, 0.372, labels.get('100111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.468, 0.708, labels.get('101000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.355, 0.572, labels.get('101001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.420, 0.679, labels.get('101010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.375, 0.597, labels.get('101011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.641, 0.436, labels.get('101100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.348, 0.538, labels.get('101101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.635, 0.453, labels.get('101110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.370, 0.548, labels.get('101111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.594, 0.689, labels.get('110000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.579, 0.670, labels.get('110001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.398, 0.670, labels.get('110010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.395, 0.653, labels.get('110011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.633, 0.682, labels.get('110100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.616, 0.656, labels.get('110101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.587, 0.427, labels.get('110110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.526, 0.415, labels.get('110111', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.495, 0.677, labels.get('111000', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.505, 0.648, labels.get('111001', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.428, 0.663, labels.get('111010', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.430, 0.631, labels.get('111011', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.639, 0.524, labels.get('111100', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.591, 0.604, labels.get('111101', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.622, 0.477, labels.get('111110', ''), fontsize=fontsize)
    draw_text(fig, ax, 0.501, 0.523, labels.get('111111', ''), fontsize=fontsize)

    # legend
    draw_text(fig, ax, 0.674, 0.824, names[0], colors[0], fontsize=fontsize)
    draw_text(fig, ax, 0.747, 0.751, names[1], colors[1], fontsize=fontsize)
    draw_text(fig, ax, 0.739, 0.396, names[2], colors[2], fontsize=fontsize)
    draw_text(fig, ax, 0.700, 0.247, names[3], colors[3], fontsize=fontsize)
    draw_text(fig, ax, 0.291, 0.255, names[4], colors[4], fontsize=fontsize)
    draw_text(fig, ax, 0.203, 0.484, names[5], colors[5], fontsize=fontsize)
    leg = ax.legend(names, loc='center left', bbox_to_anchor=(1.0, 0.5), fancybox=True)
    leg.get_frame().set_alpha(0.5)

    return fig, ax



import prody


def get_a_frame_from_dcd_to_pdb(dcd_file, pdb_file, output_pdb_prefix, index_list=None, atom_select="not water"):
    #    structure = prody.parsePDB(pdb_file).select('not water')
    structure = prody.parsePDB(pdb_file)
    ensemble = prody.parseDCD(dcd_file)
    ensemble.setAtoms(structure)
    ensemble.setCoords(structure)

    for frame in ensemble:
        if not index_list or frame.getIndex() in index_list:
            atoms = frame.getAtoms()
            coords = frame.getCoords()
            atoms.setCoords(coords)
            # prody.writePDB("%s.%d.pdb" % (output_pdb_prefix,num), atoms.select('not water'))
            prody.writePDB("%s.%d.pdb" % (output_pdb_prefix, frame.getIndex()), atoms.select(atom_select))


def atom_distance(dcd_file, pdb_file, atom_pair_list):
    structure = prody.parsePDB(pdb_file)

    ensemble = prody.parseDCD(dcd_file)
    ensemble.setAtoms(structure)
    ensemble.setCoords(structure)

    output_dir = {}
    for pair_tuple in atom_pair_list:
        output_dir[tuple(pair_tuple)] = []

    for frame in ensemble:
        atoms = frame.getAtoms()
        coords = frame.getCoords()
        # print(frame.getIndex(), prody.calcDistance(coords[2744], coords[5351]))
        for i in output_dir:
            output_dir[i].append(prody.calcDistance(coords[i[0] - 1], coords[i[1] - 1]))

    return output_dir


def get_RMSD(dcd_file, pdb_file, atom_select="not water"):
    structure = prody.parsePDB(pdb_file)
    target_atoms = structure.select(atom_select)

    ensemble = prody.parseDCD(dcd_file)
    ensemble.setAtoms(target_atoms)
    # ensemble.setCoords(target_atoms)

    # prody.calcRMSD(ensemble.getCoords())

    RMSD_list = []
    for frame in ensemble:
        atoms = frame.getAtoms()
        coords = frame.getCoords()
        if frame.getIndex() == 0:
            ref_coord = coords
            RMSD_list.append(0)
        else:
            RMSD_list.append(prody.calcRMSD(ref_coord, coords))

    return RMSD_list

"""
dcd_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190904/step1fs8000dcd20ns/minimized.dcd"
pdb_file = "/lustre/home/xuyuxing/Work/Kena/qmmm/fkn/n3_20190904/step1fs8000dcd20ns/solvate.pdb"
"""

def get_RE_RMSD(dcd_file, pdb_file, atom_select="not water"):
    structure = prody.parsePDB(pdb_file)
    ensemble = prody.parseDCD(dcd_file)

    target_atoms = structure.select(atom_select)
    ensemble.setAtoms(target_atoms)
    #ensemble.setCoords(target_atoms)

    #target_atoms = structure.select(atom_select)

    ensemble.superpose()
    rmsd_1 = ensemble.getRMSDs()
    rmsf_1 = ensemble.getRMSFs()

#########
    # structure = prody.parsePDB(pdb_file)
    # ensemble = prody.parseDCD(dcd_file)
    # ensemble.setAtoms(structure)
    # ensemble.setCoords(structure)
    #
    # target_atoms = structure.select(atom_select)
    # ensemble.setAtoms(target_atoms)
    #
    # ensemble.superpose()
    # rmsd = ensemble.getRMSDs()
    # rmsf = ensemble.getRMSFs()


    # prody.calcRMSD(ensemble.getCoords())

    # RMSD_list = []
    # for frame in ensemble:
    #     atoms = frame.getAtoms()
    #     coords = frame.getCoords()
    #     if frame.getIndex() == 0:
    #         ref_coord = coords
    #         RMSD_list.append(0)
    #     else:
    #         RMSD_list.append(prody.calcRMSD(ref_coord, coords))

    return rmsd_1,rmsf_1